#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2017 Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: iam_user
version_added: 5.0.0
short_description: Manage AWS IAM users
description:
  - A module to manage AWS IAM users.
  - The module does not manage groups that users belong to, groups memberships can be managed using M(amazon.aws.iam_group).
  - This module was originally added to C(community.aws) in release 1.0.0.
author:
  - Josh Souza (@joshsouza)
options:
  name:
    description:
      - The name of the user.
      - >-
        Note: user names are unique within an account.  Paths (O(path)) do B(not) affect
        the uniqueness requirements of O(name).  For example it is not permitted to have both
        C(/Path1/MyUser) and C(/Path2/MyUser) in the same account.
      - O(user_name) was added as an alias in release 7.2.0.
    required: true
    type: str
    aliases: ['user_name']
  path:
    description:
      - The path for the user.
      - For more information about IAM paths, see the AWS IAM identifiers documentation
        U(https://docs.aws.amazon.com/IAM/latest/UserGuide/reference_identifiers.html).
    aliases: ['prefix', 'path_prefix']
    required: false
    type: str
    version_added: 7.2.0
  boundary:
    description:
      - The ARN of an IAM managed policy to apply as a boundary policy for this user.
      - Boundary policies can be used to restrict the permissions a user can exercise, but does not
        grant any policies in and of itself.
      - For more information on boundaries, see
        U(https://docs.aws.amazon.com/IAM/latest/UserGuide/access_policies_boundaries.html).
      - Set to the empty string V("") to remove the boundary policy.
    aliases: ["boundary_policy_arn", "permissions_boundary"]
    required: false
    type: str
    version_added: 7.2.0
  password:
    description:
      - The password to apply to the user.
    required: false
    type: str
    version_added: 2.2.0
    version_added_collection: community.aws
  password_reset_required:
    description:
      - Defines if the user is required to set a new password when they log in.
      - Ignored unless a new password is set.
    required: false
    type: bool
    default: false
    version_added: 3.1.0
    version_added_collection: community.aws
  update_password:
    default: always
    choices: ['always', 'on_create']
    description:
      - When to update user passwords.
      - O(update_password=always) will ensure the password is set to O(password).
      - O(update_password=on_create) will only set the password for newly created users.
    type: str
    version_added: 2.2.0
    version_added_collection: community.aws
  remove_password:
    description:
      - Option to delete user login passwords.
      - This field is mutually exclusive to O(password).
    type: 'bool'
    version_added: 2.2.0
    version_added_collection: community.aws
  managed_policies:
    description:
      - A list of managed policy ARNs or friendly names to attach to the user.
      - To embed an inline policy, use M(community.aws.iam_policy).
    required: false
    type: list
    default: []
    elements: str
    aliases: ['managed_policy']
  state:
    description:
      - Create or remove the IAM user.
    required: true
    choices: [ 'present', 'absent' ]
    type: str
  purge_policies:
    description:
      - When O(purge_policies=true) any managed policies not listed in O(managed_policies) will be detached.
    required: false
    default: false
    type: bool
    aliases: ['purge_policy', 'purge_managed_policies']
  wait:
    description:
      - When O(wait=True) the module will wait for up to O(wait_timeout) seconds
        for IAM user creation before returning.
    default: True
    type: bool
    version_added: 2.2.0
    version_added_collection: community.aws
  wait_timeout:
    description:
      - How long (in seconds) to wait for creation / updates to complete.
    default: 120
    type: int
    version_added: 2.2.0
    version_added_collection: community.aws
notes:
  - Support for O(tags) and O(purge_tags) was added in release 2.1.0.
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.tags
  - amazon.aws.boto3
"""

EXAMPLES = r"""
# Note: These examples do not set authentication details, see the AWS Guide for details.
# Note: This module does not allow management of groups that users belong to.
#       Groups should manage their membership directly using amazon.aws.iam_group,
#       as users belong to them.

- name: Create a user
  amazon.aws.iam_user:
    name: testuser1
    state: present

- name: Create a user with a password
  amazon.aws.iam_user:
    name: testuser1
    password: SomeSecurePassword
    state: present

- name: Create a user and attach a managed policy using its ARN
  amazon.aws.iam_user:
    name: testuser1
    managed_policies:
      - arn:aws:iam::aws:policy/AmazonSNSFullAccess
    state: present

- name: Remove all managed policies from an existing user with an empty list
  amazon.aws.iam_user:
    name: testuser1
    state: present
    purge_policies: true

- name: Create user with tags
  amazon.aws.iam_user:
    name: testuser1
    state: present
    tags:
      Env: Prod

- name: Delete the user
  amazon.aws.iam_user:
    name: testuser1
    state: absent
"""

RETURN = r"""
user:
    description: Dictionary containing all the user information.
    returned: success
    type: complex
    contains:
        arn:
            description: The Amazon Resource Name (ARN) specifying the user.
            type: str
            returned: always
            sample: "arn:aws:iam::123456789012:user/testuser1"
        create_date:
            description: The date and time, in ISO 8601 date-time format, when the user was created.
            type: str
            returned: always
            sample: "2017-02-08T04:36:28+00:00"
        user_id:
            description: The stable and unique string identifying the user.
            type: str
            returned: always
            sample: "AGPA12345EXAMPLE54321"
        user_name:
            description: The friendly name that identifies the user.
            type: str
            returned: always
            sample: "testuser1"
        path:
            description: The path to the user.
            type: str
            returned: always
            sample: "/"
        tags:
            description: User tags.
            type: dict
            returned: always
            sample: {"Env": "Prod"}
        attached_policies:
            version_added: 7.2.0
            description:
                - List containing basic information about managed policies attached to the group.
            returned: always
            type: list
            elements: dict
            sample: [
                        {
                            "policy_arn": "arn:aws:iam::123456789012:policy/test_policy",
                            "policy_name": "test_policy"
                        }
                    ]
            contains:
                policy_arn:
                    description: The Amazon Resource Name (ARN) specifying the managed policy.
                    type: str
                    sample: "arn:aws:iam::123456789012:policy/test_policy"
                policy_name:
                    description: The friendly name that identifies the policy.
                    type: str
                    sample: test_policy
"""

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.iam import AnsibleIAMError
from ansible_collections.amazon.aws.plugins.module_utils.iam import IAMErrorHandler
from ansible_collections.amazon.aws.plugins.module_utils.iam import convert_managed_policy_names_to_arns
from ansible_collections.amazon.aws.plugins.module_utils.iam import get_iam_user
from ansible_collections.amazon.aws.plugins.module_utils.iam import normalize_iam_user
from ansible_collections.amazon.aws.plugins.module_utils.iam import validate_iam_identifiers
from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry
from ansible_collections.amazon.aws.plugins.module_utils.tagging import ansible_dict_to_boto3_tag_list
from ansible_collections.amazon.aws.plugins.module_utils.tagging import compare_aws_tags


@IAMErrorHandler.common_error_handler("wait for IAM user creation")
def _wait_user_exists(connection, **params):
    waiter = connection.get_waiter("user_exists")
    waiter.wait(**params)


def wait_iam_exists(connection, module):
    if not module.params.get("wait"):
        return

    user_name = module.params.get("name")
    wait_timeout = module.params.get("wait_timeout")

    delay = min(wait_timeout, 5)
    max_attempts = wait_timeout // delay
    waiter_config = {"Delay": delay, "MaxAttempts": max_attempts}

    _wait_user_exists(connection, WaiterConfig=waiter_config, UserName=user_name)


@IAMErrorHandler.common_error_handler("create user")
def create_user(connection, module, user_name, path, boundary, tags):
    params = {"UserName": user_name}
    if path:
        params["Path"] = path
    if boundary:
        params["PermissionsBoundary"] = boundary
    if tags:
        params["Tags"] = ansible_dict_to_boto3_tag_list(tags)

    if module.check_mode:
        module.exit_json(changed=True, create_params=params)

    user = connection.create_user(aws_retry=True, **params)["User"]

    return normalize_iam_user(user)


@IAMErrorHandler.common_error_handler("create user login profile")
def _create_login_profile(connection, **params):
    return connection.create_login_profile(aws_retry=True, **params)


# Uses the list error handler because we "update" as a quick test for existence
# when our next step would be update or create.
@IAMErrorHandler.list_error_handler("update user login profile")
def _update_login_profile(connection, **params):
    return connection.update_login_profile(aws_retry=True, **params)


def _create_or_update_login_profile(connection, name, password, reset):
    # Apply new password / update password for the user
    user_params = {
        "UserName": name,
        "Password": password,
        "PasswordResetRequired": reset,
    }

    retval = _update_login_profile(connection, **user_params)
    if retval:
        return retval
    return _create_login_profile(connection, **user_params)


def ensure_login_profile(connection, check_mode, user_name, password, update, reset, new_user):
    if password is None:
        return False, None
    if update == "on_create" and not new_user:
        return False, None

    if check_mode:
        return True, None

    return True, _create_or_update_login_profile(connection, user_name, password, reset)


@IAMErrorHandler.list_error_handler("get login profile")
def _get_login_profile(connection, name):
    return connection.get_login_profile(aws_retry=True, UserName=name).get("LoginProfile")


@IAMErrorHandler.deletion_error_handler("delete login profile")
def _delete_login_profile(connection, name):
    connection.delete_login_profile(aws_retry=True, UserName=name)


def remove_login_profile(connection, check_mode, user_name, remove_password, new_user):
    if new_user:
        return False
    if not remove_password:
        return False

    # In theory we could skip this check outside check_mode
    login_profile = _get_login_profile(connection, user_name)
    if not login_profile:
        return False

    if check_mode:
        return True

    _delete_login_profile(connection, user_name)
    return True


@IAMErrorHandler.list_error_handler("get policies for user")
def _list_attached_policies(connection, user_name):
    return connection.list_attached_user_policies(aws_retry=True, UserName=user_name)["AttachedPolicies"]


@IAMErrorHandler.common_error_handler("attach policy to user")
def attach_policies(connection, check_mode, user_name, policies):
    if not policies:
        return False
    if check_mode:
        return True
    for policy_arn in policies:
        connection.attach_user_policy(UserName=user_name, PolicyArn=policy_arn)


@IAMErrorHandler.common_error_handler("detach policy from user")
def detach_policies(connection, check_mode, user_name, policies):
    if not policies:
        return False
    if check_mode:
        return True
    for policy_arn in policies:
        connection.detach_user_policy(UserName=user_name, PolicyArn=policy_arn)


def ensure_managed_policies(connection, check_mode, user_name, managed_policies, purge_policies):
    if managed_policies is None:
        return False

    managed_policies = convert_managed_policy_names_to_arns(connection, managed_policies)

    # Manage managed policies
    attached_policies_desc = _list_attached_policies(connection, user_name)
    current_attached_policies = [policy["PolicyArn"] for policy in attached_policies_desc]

    policies_to_add = list(set(managed_policies) - set(current_attached_policies))
    policies_to_remove = []
    if purge_policies:
        policies_to_remove = list(set(current_attached_policies) - set(managed_policies))

    if not policies_to_add and not policies_to_remove:
        return False

    if check_mode:
        return True

    detach_policies(connection, check_mode, user_name, policies_to_remove)
    attach_policies(connection, check_mode, user_name, policies_to_add)

    return True


@IAMErrorHandler.common_error_handler("set tags for user")
def ensure_user_tags(connection, check_mode, user, user_name, new_tags, purge_tags):
    if new_tags is None:
        return False

    existing_tags = user["tags"]

    tags_to_add, tags_to_remove = compare_aws_tags(existing_tags, new_tags, purge_tags=purge_tags)

    if not tags_to_remove and not tags_to_add:
        return False

    if check_mode:
        return True

    if tags_to_remove:
        connection.untag_user(UserName=user_name, TagKeys=tags_to_remove)
    if tags_to_add:
        connection.tag_user(UserName=user_name, Tags=ansible_dict_to_boto3_tag_list(tags_to_add))

    return True


@IAMErrorHandler.deletion_error_handler("remove permissions boundary for user")
def _delete_user_permissions_boundary(connection, check_mode, user_name):
    if check_mode:
        return True
    connection.delete_user_permissions_boundary(aws_retry=True, UserName=user_name)


@IAMErrorHandler.common_error_handler("set permissions boundary for user")
def _put_user_permissions_boundary(connection, check_mode, user_name, boundary):
    if check_mode:
        return True
    connection.put_user_permissions_boundary(aws_retry=True, UserName=user_name, PermissionsBoundary=boundary)


def ensure_permissions_boundary(connection, check_mode, user, user_name, boundary):
    if boundary is None:
        return False

    current_boundary = user.get("permissions_boundary", "") if user else None

    if current_boundary:
        current_boundary = current_boundary.get("permissions_boundary_arn")

    if boundary == current_boundary:
        return False

    if check_mode:
        return True

    if boundary == "":
        _delete_user_permissions_boundary(connection, check_mode, user_name)
    else:
        _put_user_permissions_boundary(connection, check_mode, user_name, boundary)

    return True


@IAMErrorHandler.common_error_handler("set path for user")
def ensure_path(connection, check_mode, user, user_name, path):
    if path is None:
        return False

    current_path = user.get("path", "") if user else None

    if path == current_path:
        return False

    if check_mode:
        return True

    connection.update_user(aws_retry=True, UserName=user_name, NewPath=path)

    return True


def create_or_update_user(connection, module):
    user_name = module.params.get("name")

    changed = False
    new_user = False
    user = get_iam_user(connection, user_name)

    boundary = module.params.get("boundary")
    if boundary:
        boundary = convert_managed_policy_names_to_arns(connection, [module.params.get("boundary")])[0]

    if user is None:
        user = create_user(
            connection,
            module,
            user_name,
            module.params.get("path"),
            boundary,
            module.params.get("tags"),
        )
        changed = True
        # Wait for user to be fully available before continuing
        wait_iam_exists(connection, module)
        new_user = True

    profile_changed, login_profile = ensure_login_profile(
        connection,
        module.check_mode,
        user_name,
        module.params.get("password"),
        module.params.get("update_password"),
        module.params.get("password_reset_required"),
        new_user,
    )
    changed |= profile_changed

    changed |= remove_login_profile(
        connection,
        module.check_mode,
        user_name,
        module.params.get("remove_password"),
        new_user,
    )

    changed |= ensure_permissions_boundary(
        connection,
        module.check_mode,
        user,
        user_name,
        boundary,
    )

    changed |= ensure_path(
        connection,
        module.check_mode,
        user,
        user_name,
        module.params.get("path"),
    )

    changed |= ensure_managed_policies(
        connection,
        module.check_mode,
        user_name,
        module.params.get("managed_policies"),
        module.params.get("purge_policies"),
    )

    changed |= ensure_user_tags(
        connection,
        module.check_mode,
        user,
        user_name,
        module.params.get("tags"),
        module.params.get("purge_tags"),
    )

    if module.check_mode:
        module.exit_json(changed=changed)

    # Get the user again
    user = get_iam_user(connection, user_name)

    if changed and login_profile:
        # `LoginProfile` is only returned on `create_login_profile` method
        user["password_reset_required"] = login_profile.get("LoginProfile", {}).get("PasswordResetRequired", False)

    try:
        # (camel_dict_to_snake_dict doesn't handle lists, so do this as a merge of two dictionaries)
        policies = {"attached_policies": _list_attached_policies(connection, user_name)}
        user.update(camel_dict_to_snake_dict(policies))
    except AnsibleIAMError as e:
        module.warn(
            f"Failed to list attached policies - {str(e.exception)}",
        )
        pass

    module.exit_json(changed=changed, user=user)


@IAMErrorHandler.deletion_error_handler("delete access key")
def delete_access_key(connection, check_mode, user_name, key_id):
    if check_mode:
        return True
    connection.delete_access_key(aws_retry=True, UserName=user_name, AccessKeyId=key_id)
    return True


@IAMErrorHandler.list_error_handler("list access keys")
def delete_access_keys(connection, check_mode, user_name):
    access_keys = connection.list_access_keys(aws_retry=True, UserName=user_name)["AccessKeyMetadata"]
    if not access_keys:
        return False
    for access_key in access_keys:
        delete_access_key(connection, check_mode, user_name, access_key["AccessKeyId"])
    return True


@IAMErrorHandler.deletion_error_handler("delete SSH key")
def delete_ssh_key(connection, check_mode, user_name, key_id):
    if check_mode:
        return True
    connection.delete_ssh_public_key(aws_retry=True, UserName=user_name, SSHPublicKeyId=key_id)
    return True


@IAMErrorHandler.list_error_handler("list SSH keys")
def delete_ssh_public_keys(connection, check_mode, user_name):
    public_keys = connection.list_ssh_public_keys(aws_retry=True, UserName=user_name)["SSHPublicKeys"]
    if not public_keys:
        return False
    for public_key in public_keys:
        delete_ssh_key(connection, check_mode, user_name, public_key["SSHPublicKeyId"])
    return True


@IAMErrorHandler.deletion_error_handler("delete service credential")
def delete_service_credential(connection, check_mode, user_name, cred_id):
    if check_mode:
        return True
    connection.delete_ssh_public_key(aws_retry=True, UserName=user_name, SSHPublicKeyId=cred_id)
    return True


@IAMErrorHandler.list_error_handler("list service credentials")
def delete_service_credentials(connection, check_mode, user_name):
    credentials = connection.list_service_specific_credentials(aws_retry=True, UserName=user_name)[
        "ServiceSpecificCredentials"
    ]
    if not credentials:
        return False
    for credential in credentials:
        delete_service_credential(connection, check_mode, user_name, credential["ServiceSpecificCredentialId"])
    return True


@IAMErrorHandler.deletion_error_handler("delete signing certificate")
def delete_signing_certificate(connection, check_mode, user_name, cert_id):
    if check_mode:
        return True
    connection.delete_signing_certificate(aws_retry=True, UserName=user_name, CertificateId=cert_id)
    return True


@IAMErrorHandler.list_error_handler("list signing certificates")
def delete_signing_certificates(connection, check_mode, user_name):
    certificates = connection.list_signing_certificates(aws_retry=True, UserName=user_name)["Certificates"]
    if not certificates:
        return False
    for certificate in certificates:
        delete_signing_certificate(connection, check_mode, user_name, certificate["CertificateId"])
    return True


@IAMErrorHandler.deletion_error_handler("delete MFA device")
def delete_mfa_device(connection, check_mode, user_name, device_id):
    if check_mode:
        return True
    connection.deactivate_mfa_device(aws_retry=True, UserName=user_name, SerialNumber=device_id)
    return True


@IAMErrorHandler.list_error_handler("list MFA devices")
def delete_mfa_devices(connection, check_mode, user_name):
    devices = connection.list_mfa_devices(aws_retry=True, UserName=user_name)["MFADevices"]
    if not devices:
        return False
    for device in devices:
        delete_mfa_device(connection, check_mode, user_name, device["SerialNumber"])
    return True


def detach_all_policies(connection, check_mode, user_name):
    # Remove any attached policies
    attached_policies_desc = _list_attached_policies(connection, user_name)
    current_attached_policies = [policy["PolicyArn"] for policy in attached_policies_desc]
    detach_policies(connection, check_mode, user_name, current_attached_policies)


@IAMErrorHandler.deletion_error_handler("delete inline policy")
def delete_inline_policy(connection, check_mode, user_name, policy):
    if check_mode:
        return True
    connection.delete_user_policy(aws_retry=True, UserName=user_name, PolicyName=policy)
    return True


@IAMErrorHandler.list_error_handler("list inline policies")
def delete_inline_policies(connection, check_mode, user_name):
    inline_policies = connection.list_user_policies(aws_retry=True, UserName=user_name)["PolicyNames"]
    if not inline_policies:
        return False
    for policy_name in inline_policies:
        delete_inline_policy(connection, check_mode, user_name, policy_name)
    return True


@IAMErrorHandler.deletion_error_handler("remove user from group")
def remove_from_group(connection, check_mode, user_name, group_name):
    if check_mode:
        return True
    connection.remove_user_from_group(aws_retry=True, UserName=user_name, GroupName=group_name)
    return True


@IAMErrorHandler.list_error_handler("list groups containing user")
def remove_from_all_groups(connection, check_mode, user_name):
    user_groups = connection.list_groups_for_user(aws_retry=True, UserName=user_name)["Groups"]
    if not user_groups:
        return False
    for group in user_groups:
        remove_from_group(connection, check_mode, user_name, group["GroupName"])
    return True


@IAMErrorHandler.deletion_error_handler("delete user")
def delete_user(connection, check_mode, user_name):
    if check_mode:
        return True
    connection.delete_user(aws_retry=True, UserName=user_name)
    return True


def destroy_user(connection, module):
    user_name = module.params.get("name")

    user = get_iam_user(connection, user_name)
    # User is not present
    if not user:
        module.exit_json(changed=False)

    # Check mode means we would remove this user
    if module.check_mode:
        module.exit_json(changed=True)

    # Prior to removing the user we need to remove all of the related resources, or deletion will
    # fail.
    # Because policies (direct and indrect) can contain Deny rules, order is important here in case
    # we fail during deletion: lock out the user first *then* start removing policies...
    # - Prevent the user from creating new sessions
    #   - Login profile
    #   - Access keys
    #   - SSH keys
    #   - Service Credentials
    #   - Certificates
    #   - MFA Token (last so we don't end up in a state where it's possible still use password/keys)
    # - Remove policies and group membership
    #   - Managed policies
    #   - Inline policies
    #   - Group membership

    remove_login_profile(connection, module.check_mode, user_name, True, False)
    delete_access_keys(connection, module.check_mode, user_name)
    delete_ssh_public_keys(connection, module.check_mode, user_name)
    delete_service_credentials(connection, module.check_mode, user_name)
    delete_signing_certificates(connection, module.check_mode, user_name)
    delete_mfa_devices(connection, module.check_mode, user_name)
    detach_all_policies(connection, module.check_mode, user_name)
    delete_inline_policies(connection, module.check_mode, user_name)
    remove_from_all_groups(connection, module.check_mode, user_name)
    changed = delete_user(connection, module.check_mode, user_name)
    module.exit_json(changed=changed)


def main():
    argument_spec = dict(
        name=dict(required=True, type="str", aliases=["user_name"]),
        path=dict(type="str", aliases=["prefix", "path_prefix"]),
        boundary=dict(type="str", aliases=["boundary_policy_arn", "permissions_boundary"]),
        password=dict(type="str", no_log=True),
        password_reset_required=dict(type="bool", default=False, no_log=False),
        update_password=dict(default="always", choices=["always", "on_create"], no_log=False),
        remove_password=dict(type="bool", no_log=False),
        managed_policies=dict(default=[], type="list", aliases=["managed_policy"], elements="str"),
        state=dict(choices=["present", "absent"], required=True),
        purge_policies=dict(default=False, type="bool", aliases=["purge_policy", "purge_managed_policies"]),
        tags=dict(type="dict", aliases=["resource_tags"]),
        purge_tags=dict(type="bool", default=True),
        wait=dict(type="bool", default=True),
        wait_timeout=dict(default=120, type="int"),
    )

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        mutually_exclusive=[["password", "remove_password"]],
    )

    identifier_problem = validate_iam_identifiers(
        "user", name=module.params.get("name"), path=module.params.get("path")
    )
    if identifier_problem:
        module.fail_json(msg=identifier_problem)

    retry_decorator = AWSRetry.jittered_backoff(catch_extra_error_codes=["EntityTemporarilyUnmodifiable"])
    connection = module.client("iam", retry_decorator=retry_decorator)

    state = module.params.get("state")

    try:
        if state == "present":
            create_or_update_user(connection, module)
        else:
            destroy_user(connection, module)
    except AnsibleIAMError as e:
        module.fail_json_aws_error(e)


if __name__ == "__main__":
    main()



# Copyright (c) 2021 Felix Fontein
# Copyright (c) 2020 Markus Bergholz <markuman+spambelongstogoogle@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import json
import time
import typing as t
from collections.abc import Sequence
from urllib.parse import urlencode

from ansible.module_utils.common.text.converters import to_native

from ansible_collections.community.dns.plugins.module_utils._zone_record_api import (
    DNSAPIAuthenticationError,
    DNSAPIError,
)

if t.TYPE_CHECKING:
    from collections.abc import Collection  # pragma: no cover

    from ._http import HTTPHelper, HTTPMethod  # pragma: no cover

    class _RequestKwarg(t.TypedDict):  # pragma: no cover
        method: t.NotRequired[HTTPMethod]  # pragma: no cover
        headers: t.NotRequired[dict[str, str] | None]  # pragma: no cover
        data: t.NotRequired[bytes | None]  # pragma: no cover
        timeout: t.NotRequired[int | None]  # pragma: no cover


ERROR_CODES: dict[int, str] = {
    200: "Successful response",
    401: "Unauthorized",
    403: "Forbidden",
    404: "Not found",
    406: "Not acceptable",
    409: "Conflict",
    422: "Unprocessable entity",
    500: "Internal Server Error",
}
UNKNOWN_ERROR = "Unknown Error"


def _get_header_value(
    info: dict[str, t.Any],
    header_name: str,
) -> str | None:
    header_name = header_name.lower()
    header_value = info.get(header_name)
    for k, v in info.items():
        if k.lower() == header_name:
            header_value = v
    return header_value


class JSONAPIHelper:
    def __init__(
        self,
        http_helper: HTTPHelper,
        token: str,
        api: str,
        debug: bool = False,
    ) -> None:
        """
        Create a new JSON API helper instance with given API key.
        """
        self._api = api
        self._http_helper = http_helper
        self._token = token
        self._debug = debug

    def _build_url(
        self,
        url: str,
        query: dict[str, str] | Sequence[tuple[str, str]] | None = None,
    ) -> str:
        return f"{self._api}{url}{'?' + urlencode(query) if query else ''}"

    def _extract_error_message(
        self,
        result: t.Any | None,
    ) -> str:
        if result is None:
            return ""
        return f" with data: {result!r}"

    def _validate(
        self,
        *,
        result: t.Any | None = None,
        info: dict[str, t.Any],
        expected: Collection[int] | None = None,
        method: HTTPMethod = "GET",
    ) -> None:
        status = info["status"]
        url = info.get("url")  # not present when using open_url
        # Check expected status
        error_code = ERROR_CODES.get(status, UNKNOWN_ERROR)
        if expected is not None:
            if status not in expected:
                more = self._extract_error_message(result)
                statuses = ", ".join([f"{e}" for e in expected])
                raise DNSAPIError(
                    f"Expected HTTP status {statuses} for {method} {url}, but got HTTP status {status} ({error_code}){more}"
                )
        else:
            if status < 200 or status >= 300:
                more = self._extract_error_message(result)
                raise DNSAPIError(
                    f"Expected successful HTTP status for {method} {url}, but got HTTP status {status} ({error_code}){more}"
                )

    def _process_json_result(
        self,
        content: bytes | None,
        info: dict[str, t.Any],
        *,
        must_have_content: bool | Sequence[int] = True,
        method: HTTPMethod = "GET",
        expected: Collection[int] | None = None,
        require_json_object: bool = False,
    ) -> tuple[dict[str, t.Any] | list[t.Any] | None, dict[str, t.Any]]:
        if isinstance(must_have_content, Sequence):
            must_have_content = info["status"] in must_have_content
        # Check for unauthenticated
        if info["status"] == 401:
            message = "Unauthorized: the authentication parameters are incorrect (HTTP status 401)"
            try:
                body = json.loads(content.decode("utf8"))  # type: ignore
                if body["message"]:
                    message = f"{message}: {body['message']}"
            except Exception:
                pass
            raise DNSAPIAuthenticationError(message)
        if info["status"] == 403:
            message = (
                "Forbidden: you do not have access to this resource (HTTP status 403)"
            )
            try:
                body = json.loads(content.decode("utf8"))  # type: ignore
                if body["message"]:
                    message = f"{message}: {body['message']}"
            except Exception:
                pass
            raise DNSAPIAuthenticationError(message)
        # Check Content-Type header
        content_type = _get_header_value(info, "content-type")
        if content_type != "application/json" and (
            content_type is None or not content_type.startswith("application/json;")
        ):
            if must_have_content:
                raise DNSAPIError(
                    f"{method} {info['url']} did not yield JSON data, but HTTP status code {info['status']}"
                    f" with Content-Type {content_type!r} and data: {to_native(content)}"
                )
            self._validate(result=content, info=info, expected=expected, method=method)
            return None, info
        # Decode content as JSON
        try:
            result = json.loads(content.decode("utf8"))  # type: ignore
        except Exception as exc:
            # TODO: This looks like a bug; decoding errors are not "empty"!
            if must_have_content:
                raise DNSAPIError(
                    f"{method} {info['url']} did not yield JSON data, but HTTP status code {info['status']} with data: {to_native(content)}"
                ) from exc
            self._validate(result=content, info=info, expected=expected, method=method)
            return None, info
        if not isinstance(result, dict):
            if require_json_object:
                raise DNSAPIError(
                    f"{method} {info['url']} did not yield a JSON object, but {type(result)}"
                )
            if not isinstance(result, list):
                raise DNSAPIError(
                    f"{method} {info['url']} did not yield a JSON object or a JSON array, but {type(result)}"
                )
        self._validate(result=result, info=info, expected=expected, method=method)
        return result, info

    def _is_rate_limiting_result(
        self, content: bytes | None, info: dict[str, t.Any]
    ) -> bool | int | float:
        if info["status"] != 429:
            return False
        try:
            hv = _get_header_value(info, "retry-after")
            if hv is None:
                raise TypeError
            return max(min(float(hv), 60), 1)
        except (ValueError, TypeError):
            return 10

    def _should_retry(self, content: bytes | None, info: dict[str, t.Any]) -> bool:
        return info["status"] in (-1, 502, 503, 504)

    def _request(
        self, url: str, **kwargs: t.Unpack[_RequestKwarg]
    ) -> tuple[bytes | None, dict[str, t.Any]]:
        """Execute a HTTP request and handle common things like rate limiting."""
        number_retries = 10
        countdown = number_retries + 1
        cause = ""
        while True:
            content, info = self._http_helper.fetch_url(url, **kwargs)
            countdown -= 1
            retry_wait = None
            if self._should_retry(content, info):
                retry_wait = 0.5
                cause = (
                    "a local HTTP request error"
                    if info["status"] == -1
                    else f"HTTP status {info['status']}"
                )
            else:
                retry_after = self._is_rate_limiting_result(content, info)
                if retry_after is not False:
                    cause = "429 Too Many Attempts"
                    retry_wait = 10 if retry_after is True else retry_after
            if retry_wait is not None:
                if countdown <= 0:
                    break
                time.sleep(retry_wait)
                continue
            return content, info
        raise DNSAPIError(
            f"Stopping after {number_retries} failed retries with {cause}"
        )

    def _create_headers(self) -> dict[str, str]:
        return {
            "accept": "application/json",
        }

    def _create_post_headers(self) -> dict[str, str]:
        return self._create_headers()

    def _create_put_headers(self) -> dict[str, str]:
        return self._create_headers()

    @t.overload
    def _get(
        self,
        url: str,
        *,
        query: dict[str, str] | Sequence[tuple[str, str]] | None = None,
        must_have_content: t.Literal[True] = True,
        expected: Collection[int] | None = None,
        require_json_object: t.Literal[True],
    ) -> tuple[dict[str, t.Any], dict[str, t.Any]]: ...

    @t.overload
    def _get(
        self,
        url: str,
        *,
        query: dict[str, str] | Sequence[tuple[str, str]] | None = None,
        must_have_content: t.Literal[True] = True,
        expected: Collection[int] | None = None,
        require_json_object: bool = False,
    ) -> tuple[dict[str, t.Any] | list[t.Any], dict[str, t.Any]]: ...

    @t.overload
    def _get(
        self,
        url: str,
        *,
        query: dict[str, str] | Sequence[tuple[str, str]] | None = None,
        must_have_content: bool | Sequence[int] = True,
        expected: Collection[int] | None = None,
        require_json_object: t.Literal[True],
    ) -> tuple[dict[str, t.Any] | None, dict[str, t.Any]]: ...

    @t.overload
    def _get(
        self,
        url: str,
        *,
        query: dict[str, str] | Sequence[tuple[str, str]] | None = None,
        must_have_content: bool | Sequence[int] = True,
        expected: Collection[int] | None = None,
        require_json_object: bool = False,
    ) -> tuple[dict[str, t.Any] | list[t.Any] | None, dict[str, t.Any]]: ...

    def _get(
        self,
        url: str,
        *,
        query: dict[str, str] | Sequence[tuple[str, str]] | None = None,
        must_have_content: bool | Sequence[int] = True,
        expected: Collection[int] | None = None,
        require_json_object: bool = False,
    ) -> tuple[dict[str, t.Any] | list[t.Any] | None, dict[str, t.Any]]:
        full_url = self._build_url(url, query)
        if self._debug:
            pass  # pragma: no cover
            # q.q('Request: GET {0}'.format(full_url))
        headers = self._create_headers()
        content, info = self._request(full_url, headers=headers, method="GET")
        return self._process_json_result(
            content,
            info,
            must_have_content=must_have_content,
            method="GET",
            expected=expected,
            require_json_object=require_json_object,
        )

    @t.overload
    def _post(
        self,
        url: str,
        *,
        data: dict[str, t.Any] | None = None,
        query: dict[str, str] | Sequence[tuple[str, str]] | None = None,
        must_have_content: t.Literal[True] = True,
        expected: Collection[int] | None = None,
        require_json_object: t.Literal[True],
    ) -> tuple[dict[str, t.Any], dict[str, t.Any]]: ...

    @t.overload
    def _post(
        self,
        url: str,
        *,
        data: dict[str, t.Any] | None = None,
        query: dict[str, str] | Sequence[tuple[str, str]] | None = None,
        must_have_content: t.Literal[True] = True,
        expected: Collection[int] | None = None,
        require_json_object: bool = False,
    ) -> tuple[dict[str, t.Any] | list[t.Any], dict[str, t.Any]]: ...

    @t.overload
    def _post(
        self,
        url: str,
        *,
        data: dict[str, t.Any] | None = None,
        query: dict[str, str] | Sequence[tuple[str, str]] | None = None,
        must_have_content: bool | Sequence[int] = True,
        expected: Collection[int] | None = None,
        require_json_object: t.Literal[True],
    ) -> tuple[dict[str, t.Any] | None, dict[str, t.Any]]: ...

    @t.overload
    def _post(
        self,
        url: str,
        *,
        data: dict[str, t.Any] | None = None,
        query: dict[str, str] | Sequence[tuple[str, str]] | None = None,
        must_have_content: bool | Sequence[int] = True,
        expected: Collection[int] | None = None,
        require_json_object: bool = False,
    ) -> tuple[dict[str, t.Any] | list[t.Any] | None, dict[str, t.Any]]: ...

    def _post(
        self,
        url: str,
        *,
        data: dict[str, t.Any] | None = None,
        query: dict[str, str] | Sequence[tuple[str, str]] | None = None,
        must_have_content: bool | Sequence[int] = True,
        expected: Collection[int] | None = None,
        require_json_object: bool = False,
    ) -> tuple[dict[str, t.Any] | list[t.Any] | None, dict[str, t.Any]]:
        full_url = self._build_url(url, query)
        if self._debug:
            pass  # pragma: no cover
            # q.q('Request: POST {0}'.format(full_url))
        headers = self._create_post_headers()
        encoded_data = None
        if data is not None:
            headers["content-type"] = "application/json"
            encoded_data = json.dumps(data).encode("utf-8")
        content, info = self._request(
            full_url, headers=headers, method="POST", data=encoded_data
        )
        return self._process_json_result(
            content,
            info,
            must_have_content=must_have_content,
            method="POST",
            expected=expected,
            require_json_object=require_json_object,
        )

    @t.overload
    def _put(
        self,
        url: str,
        *,
        data: dict[str, t.Any] | None = None,
        query: dict[str, str] | Sequence[tuple[str, str]] | None = None,
        must_have_content: t.Literal[True] = True,
        expected: Collection[int] | None = None,
        require_json_object: t.Literal[True],
    ) -> tuple[dict[str, t.Any], dict[str, t.Any]]: ...

    @t.overload
    def _put(
        self,
        url: str,
        *,
        data: dict[str, t.Any] | None = None,
        query: dict[str, str] | Sequence[tuple[str, str]] | None = None,
        must_have_content: t.Literal[True] = True,
        expected: Collection[int] | None = None,
        require_json_object: bool = False,
    ) -> tuple[dict[str, t.Any] | list[t.Any], dict[str, t.Any]]: ...

    @t.overload
    def _put(
        self,
        url: str,
        *,
        data: dict[str, t.Any] | None = None,
        query: dict[str, str] | Sequence[tuple[str, str]] | None = None,
        must_have_content: bool | Sequence[int] = True,
        expected: Collection[int] | None = None,
        require_json_object: t.Literal[True],
    ) -> tuple[dict[str, t.Any] | None, dict[str, t.Any]]: ...

    @t.overload
    def _put(
        self,
        url: str,
        *,
        data: dict[str, t.Any] | None = None,
        query: dict[str, str] | Sequence[tuple[str, str]] | None = None,
        must_have_content: bool | Sequence[int] = True,
        expected: Collection[int] | None = None,
        require_json_object: bool = False,
    ) -> tuple[dict[str, t.Any] | list[t.Any] | None, dict[str, t.Any]]: ...

    def _put(
        self,
        url: str,
        *,
        data: dict[str, t.Any] | None = None,
        query: dict[str, str] | Sequence[tuple[str, str]] | None = None,
        must_have_content: bool | Sequence[int] = True,
        expected: Collection[int] | None = None,
        require_json_object: bool = False,
    ) -> tuple[dict[str, t.Any] | list[t.Any] | None, dict[str, t.Any]]:
        full_url = self._build_url(url, query)
        if self._debug:
            pass  # pragma: no cover
            # q.q('Request: PUT {0}'.format(full_url))
        headers = self._create_put_headers()
        encoded_data = None
        if data is not None:
            headers["content-type"] = "application/json"
            encoded_data = json.dumps(data).encode("utf-8")
        content, info = self._request(
            full_url, headers=headers, method="PUT", data=encoded_data
        )
        return self._process_json_result(
            content,
            info,
            must_have_content=must_have_content,
            method="PUT",
            expected=expected,
            require_json_object=require_json_object,
        )

    @t.overload
    def _delete(
        self,
        url: str,
        *,
        query: dict[str, str] | Sequence[tuple[str, str]] | None = None,
        must_have_content: t.Literal[True] = True,
        expected: Collection[int] | None = None,
        require_json_object: t.Literal[True],
    ) -> tuple[dict[str, t.Any], dict[str, t.Any]]: ...

    @t.overload
    def _delete(
        self,
        url: str,
        *,
        query: dict[str, str] | Sequence[tuple[str, str]] | None = None,
        must_have_content: t.Literal[True] = True,
        expected: Collection[int] | None = None,
        require_json_object: bool = False,
    ) -> tuple[dict[str, t.Any] | list[t.Any], dict[str, t.Any]]: ...

    @t.overload
    def _delete(
        self,
        url: str,
        *,
        query: dict[str, str] | Sequence[tuple[str, str]] | None = None,
        must_have_content: bool | Sequence[int] = True,
        expected: Collection[int] | None = None,
        require_json_object: t.Literal[True],
    ) -> tuple[dict[str, t.Any] | None, dict[str, t.Any]]: ...

    @t.overload
    def _delete(
        self,
        url: str,
        *,
        query: dict[str, str] | Sequence[tuple[str, str]] | None = None,
        must_have_content: bool | Sequence[int] = True,
        expected: Collection[int] | None = None,
        require_json_object: bool = False,
    ) -> tuple[dict[str, t.Any] | list[t.Any] | None, dict[str, t.Any]]: ...

    def _delete(
        self,
        url: str,
        *,
        query: dict[str, str] | Sequence[tuple[str, str]] | None = None,
        must_have_content: bool | Sequence[int] = True,
        expected: Collection[int] | None = None,
        require_json_object: bool = False,
    ) -> tuple[dict[str, t.Any] | list[t.Any] | None, dict[str, t.Any]]:
        full_url = self._build_url(url, query)
        if self._debug:
            pass  # pragma: no cover
            # q.q('Request: DELETE {0}'.format(full_url))
        headers = self._create_headers()
        content, info = self._request(full_url, headers=headers, method="DELETE")
        return self._process_json_result(
            content,
            info,
            must_have_content=must_have_content,
            method="DELETE",
            expected=expected,
            require_json_object=require_json_object,
        )



#!/usr/bin/python
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: azure_rm_ml_batch_endpoint_invoke
version_added: "3.18.0"
short_description: Invoke ML Batch Endpoint resources
description:
    - Invoke ML Batch Endpoint resources.
options:
    name:
        description:
            - Name of the Azure ML batch endpoint.
        type: str
        required: false
    resource_group:
        description:
            - Name of resource group.
        required: true
        type: str
    ml_workspace:
        description:
            - Name of the Azure ML workspace.
        required: true
        type: str
    invocation_definition:
        description:
            - Batch Endpoint Invocations definition in YAML.
        required: false
        type: str
    deployment_name:
        description:
            - Name of the deployment to target.
        required: false
        type: str
    experiment_name:
        description:
            - Name of the experiment for pipeline component deployment.
        required: false
        type: str
    input:
        description:
            - Reference to input data to use for batch inferencing.
              It can be a path on the datastore, public URI, a registered
              data asset, or a local folder path.
        required: false
        type: str
    input_type:
        description:
            - Type of the input, specifying whether it's a file or a folder.
              Use this when you are using a path on datastore or public URI.
        required: false
        type: str
        choices:
            - uri_folder
            - uri_file
    inputs:
        description:
            - Dictionary of Inputs of invoke jobs.
        required: false
        type: dict
    instance_count:
        description:
            - Number of instances the prediction will run on.
        required: false
        type: int
    job_name:
        description:
            - Name of the job for batch invoke.
        required: false
        type: str
    mini_batch_size:
        description:
            - Size of each mini batch that the input data will be split into
              for prediction.
        required: false
        type: int
    output_path:
        description:
            - Path on the datastore where output files will be uploaded to.
        required: false
        type: str
    outputs:
        description:
            - Dictionary to specify where to store the results.
        required: false
        type: dict

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Bill Peck (@p3ck)
'''

EXAMPLES = '''
- name: Invoke ML Batch Endpoint
  azure.azcollection.azure_rm_ml_batch_endpoint_invoke:
    resource_group: myResourceGroup
    ml_workspace: myMLWorkspace
'''

RETURN = '''
changed:
    description:
        - Will return True if successfully invoked
    returned: always
    type: bool
    sample: true
ml_batch_endpoint_invoke:
    description:
        - Batch Endpoints that match the query.
    returned: always
    type: dict
    sample: {
        "id": "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/xxxxx-ml-batch_endpoint_invoke/providers/Microsoft.MachineLearningServices/workspaces/workspace-xxxxxxxxxx/batchEndpoints//jobs/pipelinejob-xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
        "name": "pipelinejob-xxxxxxxx-xxxx-xxxxxxxxx-xxxxxxxxxxxx",
        "properties": {
            "compute": {
                "is_local": false
            },
            "interaction_endpoints": {
                "Studio": {
                    "endpoint": "https://ml.azure.com/runs/pipelinejob-xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx?wsid=/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourcegroups/xxxxx-ml-batch_endpoint_invoke/workspaces/workspace-xxxxxxxxxx",
                    "job_endpoint_type": "Studio"
                },
                "Tracking": {
                    "endpoint": "azureml://eastus.api.azureml.ms/mlflow/v1.0/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/xxxxx-ml-batch_endpoint_invoke/providers/Microsoft.MachineLearningServices/workspaces/workspace-xxxxxxxxxx?",
                    "job_endpoint_type": "Tracking"
                }
            },
            "name": "bold_egg_xxxxxxxx",
            "output": {},
            "properties": {
                "azureml.DatasetAccessMode": "Asset",
                "azureml.DevPlatv2": "true",
                "azureml.SourceComponentId": "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/xxxxx-ml-batch_endpoint_invoke/providers/Microsoft.MachineLearningServices/workspaces/workspace-xxxxxxxxxx/components/hello_batch/versions/1",
                "azureml.continue_on_failed_optional_input": "True",
                "azureml.continue_on_step_failure": "True",
                "azureml.defaultComputeName": "batch-cluster",
                "azureml.defaultDataStoreName": "workspaceblobstore",
                "azureml.deploymentname": "deployment-xxxxxxxxxx-ansible",
                "azureml.endpointname": "endpoint-xxxxxxxxxx-ansible",
                "azureml.enforceRerun": "False",
                "azureml.parameters": "{}",
                "azureml.pipelineComponent": "pipelinerun",
                "azureml.runLineageType": "batchDeployment",
                "azureml.runsource": "azureml.PipelineRun",
                "azureml.sourcepipelinerunid": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                "runSource": "BatchDeployment",
                "runType": "HTTP"
            },
            "provisioning_state": "Succeeded",
            "status": "Preparing",
            "tags": {
                "azureml.batchrun": "true",
                "azureml.deploymentname": "deployment-xxxxxxxxxx-ansible",
                "azureml.jobtype": "azureml.pipelinejob"
            }
        },
        "system_data": {
            "created_at": "2026-05-04T13:56:02.890323Z",
            "created_by": "xxxx xxxx-xxxxxxxxxx",
            "last_modified_at": "2026-05-04T13:56:02.890323Z",
            "last_modified_by": "xxxx xxxx-xxxxxxxxxx"
        },
        "type": "Microsoft.MachineLearningServices/workspaces/batchEndpoints/jobs"
    }
'''  # NOQA


try:
    import io
    from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ml import MLClientCommon
    from azure.core.exceptions import ResourceNotFoundError
    from azure.ai.ml.constants._endpoint import BatchEndpointInvoke, EndpointYamlFields
    from azure.ai.ml.constants._common import ARM_ID_PREFIX, AssetTypes, InputTypes
    from azure.ai.ml.entities._inputs_outputs import Input, Output
    from azure.ai.ml.entities._load_functions import _try_load_yaml_dict
    from azure.ai.ml._utils._arm_id_utils import parse_name_version, remove_aml_prefix
    from azure.ai.ml._utils._storage_utils import AzureMLDatastorePathUri
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMMLBatchEndpointInvoke(MLClientCommon):
    def __init__(self):

        self.module_arg_spec = dict(
            name=dict(
                type='str',
                required=False,
            ),
            resource_group=dict(
                type='str',
                required=True,
            ),
            ml_workspace=dict(
                type='str',
                required=True,
            ),
            invocation_definition=dict(
                type='str',
                required=False,
            ),
            deployment_name=dict(
                type='str',
                required=False,
            ),
            experiment_name=dict(
                type='str',
                required=False,
            ),
            input=dict(
                type='str',
                required=False,
            ),
            input_type=dict(
                type='str',
                required=False,
                choices=['uri_folder', 'uri_file'],
            ),
            inputs=dict(
                type='dict',
                required=False,
            ),
            instance_count=dict(
                type='int',
                required=False,
            ),
            job_name=dict(
                type='str',
                required=False,
            ),
            mini_batch_size=dict(
                type='int',
                required=False,
            ),
            output_path=dict(
                type='str',
                required=False,
            ),
            outputs=dict(
                type='dict',
                required=False,
            ),
        )

        self._client = None
        self.ml_registry = None

        self.results = dict(
            ml_batch_endpoint_invoke=None,
            changed=False,
        )

        required_if = [
            ('invocation_definition', None, ['name'])
        ]

        super(AzureRMMLBatchEndpointInvoke, self).__init__(self.module_arg_spec,
                                                           supports_tags=False,
                                                           supports_check_mode=False,
                                                           facts_module=True,
                                                           required_if=required_if,
                                                           )

    def exec_module(self, **kwargs):

        for key in list(self.module_arg_spec.keys()):
            setattr(self, key, kwargs[key])

        params_override = []
        input = None
        inputs = None
        outputs = None

        try:
            if self.deployment_name:
                self.client.batch_endpoints._validate_deployment_name(
                    self.name, self.deployment_name
                )

            if self.invocation_definition:
                invocation_definition = io.StringIO(self.invocation_definition)
                batch_endpoint_invocation = _try_load_yaml_dict(invocation_definition)

                inputs_dict = batch_endpoint_invocation.get(BatchEndpointInvoke.INPUTS)
                if inputs_dict:
                    inputs = _parse_inputs(inputs_dict)
                outputs_dict = batch_endpoint_invocation.get(BatchEndpointInvoke.OUTPUTS)
                if outputs_dict:
                    outputs = {}
                    for key, data in outputs_dict.items():
                        output = Output(
                            type=data.get(BatchEndpointInvoke.TYPE),
                            path=data.get(BatchEndpointInvoke.PATH),
                            mode=data.get(BatchEndpointInvoke.MODE),
                        )
                        outputs[key] = output
                if not self.deployment_name:
                    self.deployment_name = batch_endpoint_invocation.get(BatchEndpointInvoke.DEPLOYMENT)
                if not self.name:
                    self.name = batch_endpoint_invocation.get(BatchEndpointInvoke.ENDPOINT)
            elif self.input or self.input_type:
                if self.input and not self.input_type:
                    if self.input.startswith(ARM_ID_PREFIX):  # azureml:data:version
                        self.input = remove_aml_prefix(self.input)
                        asset_name, asset_version = parse_name_version(self.input)
                        try:
                            data = self.client.data.get(asset_name, asset_version)
                            # invoke input should be of type Input based on the data type
                            if data.type == AssetTypes.URI_FOLDER:
                                input = Input(type=AssetTypes.URI_FOLDER, path=data.id)
                            elif data.type == AssetTypes.URI_FILE:
                                input = Input(type=AssetTypes.URI_FILE, path=data.id)
                        except ResourceNotFoundError:
                            # We have a case that the input is not a registered data asset
                            self.fail(
                                "Value passed to input is not a supported data. "
                                "Please use an Azure Machine Learning registered V2 data asset "
                                "(with the type of either `uri_file` or `uri_folder`). Refer to how-to guide for "
                                "batch endpoint for more information on supported input data types."
                            )
                    else:  # ./path
                        input = Input(type=AssetTypes.URI_FOLDER, path=self.input)
                elif self.input_type == AssetTypes.URI_FOLDER:
                    input = Input(type=AssetTypes.URI_FOLDER, path=self.input)
                elif self.input_type == AssetTypes.URI_FILE:
                    input = Input(type=AssetTypes.URI_FILE, path=self.input)
                else:
                    self.fail(
                        "Unsupported input please use either a path on the datastore, "
                        "public URI, a registered data asset, or a local folder path."
                    )
            properties = {}
            if self.mini_batch_size:
                params_override.append({EndpointYamlFields.MINI_BATCH_SIZE: self.mini_batch_size})
            if self.instance_count:
                params_override.append({EndpointYamlFields.BATCH_JOB_INSTANCE_COUNT: self.instance_count})
            if self.output_path:
                try:
                    output = validate_and_split_output_path(self.output_path)
                except ValueError as err:
                    self.fail(err)
                params_override.append(
                    {EndpointYamlFields.BATCH_JOB_OUTPUT_PATH: output.path}
                )
                params_override.append(
                    {EndpointYamlFields.BATCH_JOB_OUTPUT_DATSTORE: "azureml:" + output.datastore}
                )
            if self.job_name:
                params_override.append(
                    {EndpointYamlFields.BATCH_JOB_NAME: self.job_name}
                )
            if self.experiment_name:
                properties["experimentName"] = self.experiment_name
            if properties:
                params_override.append(
                    {EndpointYamlFields.BATCH_JOB_PROPERTIES: properties}
                )
            if self.inputs:
                inputs = _parse_inputs(self.inputs)
            if self.outputs:
                outputs = {}
                for key, data in self.outputs.items():
                    output = Output(
                        type=data.get(BatchEndpointInvoke.TYPE),
                        path=data.get(BatchEndpointInvoke.PATH),
                        mode=data.get(BatchEndpointInvoke.MODE),
                    )
                    outputs[key] = output

            ml_batch_endpoint_invoke = self.client.batch_endpoints.invoke(
                endpoint_name=self.name,
                input=input,
                inputs=inputs,
                outputs=outputs,
                deployment_name=self.deployment_name,
                params_override=params_override,
            )

        except Exception as err:
            if isinstance(err, ResourceNotFoundError) and self.deployment_name is None:
                self.fail("Set a default deployment or provide a deployment name.")
            else:
                self.fail(err)

        self.results['ml_batch_endpoint_invoke'] = self.entity_to_dict(
            ml_batch_endpoint_invoke
        )
        self.results['changed'] = True
        return self.results

    def get(self, name):
        try:
            result = self.client.batch_endpoints.get(name=name)
            ml_batch_endpoint = self.entity_to_dict(result)
            jobs = self.client.batch_endpoints.list_jobs(endpoint_name=name)
            ml_batch_endpoint['jobs'] = jobs
        except ResourceNotFoundError:
            ml_batch_endpoint = None

        return ml_batch_endpoint


def _parse_inputs(inputs_dict):
    inputs = {}
    if inputs_dict:
        for key, data in inputs_dict.items():
            if isinstance(data, str):
                inputs[key] = Input(type=InputTypes.STRING, default=data)
            elif isinstance(data, bool):
                inputs[key] = Input(type=InputTypes.BOOLEAN, default=data)
            elif isinstance(data, int):
                inputs[key] = Input(type=InputTypes.INTEGER, default=data)
            elif isinstance(data, float):
                inputs[key] = Input(type=InputTypes.NUMBER, default=data)
            else:
                if data.get(BatchEndpointInvoke.TYPE) in [InputTypes.NUMBER, InputTypes.INTEGER]:
                    inputs[key] = Input(
                        type=data.get(BatchEndpointInvoke.TYPE, None),
                        default=data.get(BatchEndpointInvoke.DEFAULT, None),
                        min=data.get(BatchEndpointInvoke.MIN, None),
                        max=data.get(BatchEndpointInvoke.MAX, None),
                    )
                elif data.get(BatchEndpointInvoke.TYPE) in [InputTypes.STRING, InputTypes.BOOLEAN]:
                    inputs[key] = Input(
                        type=data.get(BatchEndpointInvoke.TYPE, None),
                        default=data.get(BatchEndpointInvoke.DEFAULT, None),
                    )
                else:
                    inputs[key] = Input(
                        type=data.get(BatchEndpointInvoke.TYPE, None),
                        path=data.get(BatchEndpointInvoke.PATH, None),
                        mode=data.get(BatchEndpointInvoke.MODE, None),
                    )
    return inputs


def validate_and_split_output_path(output_path):
    if output_path.startswith(ARM_ID_PREFIX):
        datastore_path = AzureMLDatastorePathUri(output_path)
        return datastore_path
    raise ValueError("Not a valid output_path, it should start with 'azureml:'")


def main():
    AzureRMMLBatchEndpointInvoke()


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# Copyright (c) 2026 Zun Yang (@zunyangc)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_appconfiguration_info
version_added: "3.15.0"
short_description: Get Azure App Configuration store facts
description:
    - Get facts for Azure App Configuration stores (Microsoft.AppConfiguration/configurationStores).
    - Supports get by name, list by resource group and list by subscription.
    - Optionally includes store replicas when fetching by name.

options:
    resource_group:
        description:
            - The name of the resource group to which the App Configuration store belongs.
        type: str
    appconfig_name:
        description:
            - The name of the App Configuration store.
        type: str
    include_replicas:
        description:
            - When C(appconfig_name) is specified, indicates whether to include the store replicas in the response.
            - Disabled for list operations to avoid extra calls by default.
        type: bool
        default: false
    tags:
        description:
            - Limit results by a list of tags. Format tags as C(key) or C(key:value).
        type: list
        elements: str

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Zun Yang (@zunyangc)
'''

EXAMPLES = '''
- name: Get App Configuration store
  azure_rm_appconfiguration_info:
    resource_group: myRG
    appconfig_name: myAGS

- name: Get App Configuration store with replicas
  azure_rm_appconfiguration_info:
    resource_group: myRG
    appconfig_name: myAGS
    include_replicas: true

- name: List stores in RG
  azure_rm_appconfiguration_info:
    resource_group: myRG

- name: List stores in subscription
  azure_rm_appconfiguration_info:
'''

RETURN = '''
appconfigurations:
    description: List of App Configuration stores.
    returned: always
    type: list
    contains:
        appconfig_name:
            description:
                - Name of the App Configuration store.
            returned: always
            type: str
            sample: myAGS
        id:
            description:
                - Resource ID of the App Configuration store.
            returned: always
            type: str
            sample: /subscriptions/xxxx/resourceGroups/myRG/providers/Microsoft.AppConfiguration/configurationStores/myAGS
        location:
            description:
                - Azure location of the App Configuration store.
            returned: always
            type: str
            sample: eastus
        tags:
            description:
                - List of tags.
            type: dict
            sample: "{'env': 'prod', 'department': 'finance'}"
        sku:
            description:
                - SKU of the App Configuration store.
            returned: always
            type: dict
            contains:
                name:
                    description: SKU name.
                    type: str
                    returned: always
                    sample: Standard
        public_network_access:
            description:
                - Whether public network access is enabled.
            returned: always
            type: str
            sample: Enabled
        disable_local_auth:
            description:
                - Whether local authentication is disabled.
            returned: always
            type: bool
            sample: false
        replicas:
            description:
                - List of replicas for the App Configuration store.
            returned: when include_replicas is true and appconfig_name is specified
            type: list
            elements: dict
            contains:
                name:
                    description:
                        - Name of the replica.
                    returned: always
                    type: str
                    sample: myAGSrep
                location:
                    description:
                        - Azure location of the replica.
                    returned: always
                    type: str
                    sample: westus
                id:
                    description:
                        - Resource ID of the replica.
                    returned: always
                    type: str
                    sample: /subscriptions/xxxx/resourceGroups/myrg/providers/Microsoft.AppConfiguration/configurationStores/myAGS/replicas/myAGSrep
        soft_delete_retention_in_days:
            description:
                - Number of days to retain soft-deleted items.
            returned: always
            type: int
            sample: 7
        enable_purge_protection:
            description:
                - Whether purge protection is enabled.
            returned: always
            type: bool
            sample: true
        identity:
            description:
                - Managed identity information.
            returned: always
            type: dict
            contains:
                type:
                    description: Type of managed identity.
                    type: str
                    returned: always
                    sample: SystemAssigned
                principal_id:
                    description: Principal ID of the managed identity.
                    type: str
                    returned: always
                    sample: xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
                tenant_id:
                    description: Tenant ID of the managed identity.
                    type: str
                    returned: always
                    sample: xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
'''


from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase

try:
    from azure.mgmt.appconfiguration import AppConfigurationManagementClient
    from azure.core.exceptions import ResourceNotFoundError
except ImportError:
    # This is handled in azure_rm_common
    pass


def appconfig_to_dict(appconfig, replicas=None, include_replicas=False):
    s = appconfig.as_dict()

    data = dict(
        id=s.get("id"),
        appconfig_name=s.get("name"),
        location=s.get("location"),
        tags=s.get("tags"),
        sku=dict(name=s.get("sku", {}).get("name")),

        endpoint=s.get("endpoint"),
        public_network_access=s.get("public_network_access"),
        disable_local_auth=s.get("disable_local_auth"),
        enable_purge_protection=s.get("enable_purge_protection"),
        soft_delete_retention_in_days=s.get("soft_delete_retention_in_days"),
        identity=s.get("identity")
    )

    # Only include replicas when explicitly requested
    if include_replicas:
        data["replicas"] = replicas or []

    return data


class AzureRMAppConfigurationInfo(AzureRMModuleBase):
    def __init__(self):
        self.module_arg_spec = dict(
            resource_group=dict(type='str'),
            appconfig_name=dict(type='str'),
            include_replicas=dict(type='bool', default=False),
            tags=dict(type='list', elements='str'),
        )

        self.resource_group = None
        self.appconfig_name = None
        self.include_replicas = None
        self.tags = None
        self.results = dict(changed=False)
        self._client = None

        super(AzureRMAppConfigurationInfo, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                          supports_check_mode=True,
                                                          supports_tags=False,
                                                          facts_module=True)

    def exec_module(self, **kwargs):
        """Main module execution method"""
        for key in list(self.module_arg_spec.keys()) + ['tags']:
            if hasattr(self, key):
                setattr(self, key, kwargs[key])

        self._client = self.get_mgmt_svc_client(AppConfigurationManagementClient,
                                                base_url=self._cloud_environment.endpoints.resource_manager)

        if self.appconfig_name:
            if not self.resource_group:
                self.fail("Parameter 'resource_group' is required when filtering by appconfig_name.")
            self.results['appconfigurations'] = [self.get_by_appconfig_name()]

        elif self.resource_group:
            self.results['appconfigurations'] = self.list_appconfig_by_resource_group()

        else:
            self.results["appconfigurations"] = self.list_appconfig_by_subscription()

        return self.results

    def get_by_appconfig_name(self):
        '''
        Get the properties of this specified app configuration store.

        :return: deserialized app configuration store state dictionary.
        '''
        try:
            s = self._client.configuration_stores.get(resource_group_name=self.resource_group,
                                                      config_store_name=self.appconfig_name)
            replicas = self.list_appconfig_replicas(self.resource_group, self.appconfig_name) if self.include_replicas else []
            if self.has_tags(s.tags, self.tags):
                return appconfig_to_dict(s, replicas, self.include_replicas)
        except ResourceNotFoundError as e:
            self.log("Did not find the app configurations {0}: {1}".format(self.appconfig_name, str(e)))

    def list_appconfig_by_resource_group(self):
        '''
        List the properties of app configuration store in specific resource group.

        :return: deserialized app configuration store state dictionary.
        '''
        results = []
        try:
            stores = list(self._client.configuration_stores.list_by_resource_group(resource_group_name=self.resource_group))

            if stores:
                for s in stores:
                    if not self.has_tags(s.tags, self.tags):
                        continue

                    replicas = self.list_appconfig_replicas(self.resource_group, s.name) if self.include_replicas else []
                    results.append(appconfig_to_dict(s, replicas, self.include_replicas))
        except Exception as e:
            self.log("Did not find appconfig in resource group {0} : {1}.".format(self.resource_group, str(e)))

        return results

    def list_appconfig_by_subscription(self):
        '''
        List the properties of app configuration stores in specific subscription.

        :return: deserialized app configuration stores state dictionary.
        '''
        results = []
        try:
            stores = list(self._client.configuration_stores.list())

            for s in stores:
                if not self.has_tags(s.tags, self.tags):
                    continue

                # parse ID to retrieve RG + name
                parts = s.id.split("/")
                rg = parts[4]
                name = parts[8]

                replicas = self.list_appconfig_replicas(rg, name) if self.include_replicas else []

                store_obj = self._client.configuration_stores.get(rg, name)
                results.append(appconfig_to_dict(store_obj, replicas, self.include_replicas))
        except Exception as e:
            self.log("Did not find appconfig in current subscription {0}.", format(str(e)))
        return results

    def list_appconfig_replicas(self, rg, name):
        '''
        List replicas of the specific app configuration stores

        :return: name, id and location of the replicas
        '''
        rlist = []
        pager = self._client.replicas.list_by_configuration_store(resource_group_name=rg,
                                                                  config_store_name=name)
        for r in pager:
            rlist.append(dict(id=r.id,
                              name=r.name,
                              location=r.location))
        return rlist


def main():
    """Main execution"""
    AzureRMAppConfigurationInfo()


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# Copyright (c) 2024 xuzhang3 (@xuzhang3), Fred-sun (@Fred-sun)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_virtualnetworkgatewayconnection_info

version_added: "2.7.0"

short_description: Gets or list the specified virtual network gateway connection

description:
    - Gets or list the specified virtual network gateway connection.

options:
    resource_group:
        description:
            - The name of the resource group.
        type: str
        required: true
    name:
        description:
            - The name of the virtual network gateway connection.
        type: str
    tags:
        description:
            - Limit results by providing a list of tags. Format tags as 'key' or 'key:value'.
        type: list
        elements: str

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - xuzhang3 (@xuzhang3)
    - Fred Sun (@Fred-sun)
'''

EXAMPLES = '''
- name: Gets the specified local network gateway connection by name
  azure_rm_virtualnetworkgatewayconnection_info:
    resource_group: "{{ resource_group }}"
    name: "{{ local_networkgateway_name }}"

- name: Gets all the virtual network gateway connection in a resource group
  azure_rm_virtualnetworkgatewayconnection_info:
    resource_group: "{{ resource_group }}"

- name: Gets all the virtual network gateway connection in a resource group and filter by tags
  azure_rm_virtualnetworkgatewayconnection_info:
    resource_group: "{{ resource_group }}"
    tags:
      - foo
'''

RETURN = '''
state:
    description:
        - Current state of the Azure Virtual Network Gateway Connection resource.
    returned: always
    type: complex
    contains:
        id:
            description:
                - The resource ID.
            type: str
            returned: always
            sample: "/subscriptions/xxx-xxx/resourceGroups/rg/providers/Microsoft.Network/connections/fredvnet"
        authorization_key:
            description:
                - The authorizationKey.
            type: str
            returned: always
            sample: "308201E806092A864886F*************B9FADDAC2D"
        connection_mode:
            description:
                - The connection mode for this connection.
            type: str
            returned: always
            sample: Default
        connection_protocol:
            description:
                - Connection protocol used for this connection.
            type: str
            returned: always
            sample: IKEv2
        connection_type:
            description:
                - Gateway connection type.
            type: str
            returned: always
            sample: Vnet2Vnet
        dpd_timeout_seconds:
            description:
                - The dead peer detection timeout of this connection in seconds.
            type: int
            returned: always
            sample: 45
        enable_bgp:
            description:
                - EnableBgp flag.
            type: bool
            returned: always
            sample: false
        etag:
            description:
                - A unique read-only string that changes whenever the resource is updated.
            type: str
            returned: always
            sample: 58b6f0c2-7d7c-4666-a62e-6416b4e00c83
        express_route_gateway_bypass:
            description:
                - Bypass ExpressRoute Gateway for data forwarding.
            type: bool
            returned: always
            sample: false
        ipsec_policies:
            description:
                - The IPSec Policies to be considered by this connection.
            type: list
            returned: always
            sample: []
        location:
            description:
                - The resource location.
            type: str
            returned: always
            sample: eastus
        name:
            description:
                - The resource name.
            type: str
            returned: always
            sample: fredvnetconnection
        provisioning_state:
            description:
                - The provisioning state of the virtual network gateway connection resource.
            type: str
            returned: always
            sample: Succeeded
        resource_group:
            description:
                - The resource group of the virtual network gateway connection.
            type: str
            returned: always
            sample: rg
        routing_weight:
            description:
                - The routing weight.
            type: int
            returned: always
            sample: 0
        shared_key:
            description:
                - The IPSec shared key.
            type: str
            returned: always
            sample: null
        tags:
            description:
                - The resource tags.
            type: dict
            returned: always
            sample: {'key1': 'value1'}
        traffic_selector_policies:
            description:
                - The Traffic Selector Policies to be considered by this connection.
            type: list
            returned: always
            sample: []
        use_local_azure_ip_address:
            description:
                - Use private local Azure IP for the connection.
            type: bool
            returned: always
            sample: false
        use_policy_based_traffic_selectors:
            description:
                - Enable policy-based traffic selectors.
            type: bool
            returned: always
            sample: false
        local_network_gateway2:
            description:
                - The reference to local network gateway resource.
            type: complex
            returned: when-used
            contains:
                id:
                    description:
                        - The ID of the local network gateway resource.
                    type: str
                    returned: always
                    sample: /subscriptions/xxx-xxx/resourceGroups/rg/providers/Microsoft.Network/localNetworkGateways/rpfx001
        virtual_network_gateway1:
            description:
                - The reference to virtual network gateway resource.
            type: complex
            returned: always
            contains:
                id:
                    description:
                        - The ID of the virtul network gateway resource.
                    type: str
                    returned: always
                    sample: /subscriptions/xxx-xxx/resourceGroups/rg/providers/Microsoft.Network/virtualNetworkGateways/fredvng
        virtual_network_gateway2:
            description:
                - The reference to virtual network gateway resource.
            type: complex
            returned: always
            contains:
                id:
                    description:
                        - The ID of the virtul network gateway resource.
                    type: str
                    returned: always
                    sample: /subscriptions/xxx-xxx/resourceGroups/rg/providers/Microsoft.Network/virtualNetworkGateways/fredvng02

'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase

try:
    from azure.core.exceptions import HttpResponseError
except Exception:
    # handled in azure_rm_common
    pass


class AzureRMVirutalNetworkGatewayConnectionInfo(AzureRMModuleBase):
    """Gets or list the specified virtual network gateway connection facts"""

    def __init__(self):

        self.module_args = dict(
            name=dict(type='str'),
            resource_group=dict(type='str', required=True),
            tags=dict(type='list', elements='str'),
        )

        self.name = None
        self.tags = None
        self.resource_group = None

        self.results = dict(
            changed=False,
            state=[],
        )

        super(AzureRMVirutalNetworkGatewayConnectionInfo, self).__init__(derived_arg_spec=self.module_args,
                                                                         supports_check_mode=True,
                                                                         supports_tags=False,
                                                                         facts_module=True)

    def exec_module(self, **kwargs):

        for key in self.module_args:
            setattr(self, key, kwargs[key])

        if self.name is not None:
            self.results['state'] = self.get_network_gateway_connection()
        else:
            self.results['state'] = self.list_network_gateway_connection()

        return self.results

    def get_network_gateway_connection(self):
        """Gets the specified local network gateway connection in a resource group"""
        response = None

        try:
            response = self.network_client.virtual_network_gateway_connections.get(self.resource_group, self.name)
        except HttpResponseError as ec:
            self.log("Gets the specified local network gateway connection in a resource group Failed, Exception as {0}".format(ec))
        if response and self.has_tags(response.tags, self.tags):
            return [self.format_response(response)]
        else:
            return []

    def list_network_gateway_connection(self):
        """Gets all the virtual network gateway connection in a resource group"""
        response = None

        try:
            response = self.network_client.virtual_network_gateway_connections.list(self.resource_group)
        except HttpResponseError as ec:
            self.log("Gets all the virtual network gateway connection in a resource group Failed, Exception as {0}".format(ec))

        if response:
            results = []
            for item in response:
                if self.has_tags(item.tags, self.tags):
                    results.append(self.format_response(item))
            return results
        else:
            return []

    def format_response(self, vngwconn):

        result = dict(
            resource_group=self.resource_group,
            authorization_key=vngwconn.authorization_key,
            connection_mode=vngwconn.connection_mode,
            connection_protocol=vngwconn.connection_protocol,
            connection_type=vngwconn.connection_type,
            dpd_timeout_seconds=vngwconn.dpd_timeout_seconds,
            etag=vngwconn.etag,
            enable_bgp=vngwconn.enable_bgp,
            express_route_gateway_bypass=vngwconn.express_route_gateway_bypass,
            id=vngwconn.id,
            ipsec_policies=vngwconn.ipsec_policies,
            location=vngwconn.location,
            name=vngwconn.name,
            provisioning_state=vngwconn.provisioning_state,
            routing_weight=vngwconn.routing_weight,
            tags=vngwconn.tags,
            traffic_selector_policies=vngwconn.traffic_selector_policies,
            shared_key=vngwconn.shared_key,
            use_local_azure_ip_address=vngwconn.use_local_azure_ip_address,
            use_policy_based_traffic_selectors=vngwconn.use_policy_based_traffic_selectors,
            virtual_network_gateway1=None,
            virtual_network_gateway2=None,
            local_network_gateway2=None
        )
        if vngwconn.local_network_gateway2:
            result['local_network_gateway2'] = dict(id=vngwconn.local_network_gateway2.id)
        if vngwconn.virtual_network_gateway1:
            result['virtual_network_gateway1'] = dict(id=vngwconn.virtual_network_gateway1.id)
        if vngwconn.virtual_network_gateway2:
            result['virtual_network_gateway2'] = dict(id=vngwconn.virtual_network_gateway2.id)

        return result


def main():
    """Main module execution code path"""

    AzureRMVirutalNetworkGatewayConnectionInfo()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright: (c) 2024, Samita Bhattacharjee (@samitab) <samitab.cisco.com>

# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "certified"}

DOCUMENTATION = r"""
---
module: aci_fabric_external_connection_profile
short_description: Manage Fabric External Connection Profiles (fv:FabricExtConnP).
description:
- Manage Fabric External Connection Profiles (Intrasite/Intersite profiles) on Cisco ACI fabrics.
options:
  description:
    description:
    - Specifies a description of the profile definition.
    type: str
    aliases: [ descr ]
  fabric_id:
    description:
    - The fabric identifier of the Fabric External Connection Profile.
    type: int
    aliases: [ id, fabric ]
  name:
    description:
    - The name of the Fabric External Connection Profile.
    type: str
    aliases: [ profile_name ]
  community:
    description:
    - Global EVPN Route Target of the Fabric External Connection Profile.
    - eg. extended:as2-nn4:5:16
    type: str
    aliases: [ rt, route_target ]
  site_id:
    description:
    - The site identifier of the Fabric External Connection Profile.
    type: int
    aliases: [ sid, site, s_id ]
  peering_type:
    description:
    - The BGP EVPN Peering Type. Use either C(automatic_with_full_mesh) or C(automatic_with_rr).
    type: str
    choices: [ automatic_with_full_mesh, automatic_with_rr ]
    aliases: [ p_type, peer, peer_t ]
  peering_password:
    description:
    - The BGP EVPN Peering Password. Used for setting automatic peering sessions.
    - Providing this option will always result in a change because it is a secure property that cannot be retrieved from APIC.
    type: str
    aliases: [ peer_password, peer_pwd ]
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation
- cisco.aci.owner

seealso:
- name: APIC Management Information Model reference
  description: More information about the internal APIC class B(fv:FabricExtConnP).
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Samita Bhattacharjee (@samitab)
"""

EXAMPLES = r"""
- name: Add a new Fabric External Connection Profile
  cisco.aci.aci_fabric_external_connection_profile:
    host: apic
    username: admin
    password: SomeSecretPassword
    fabric_id: 1
    name: ansible_fabric_ext_conn_profile
    description: Fabric External Connection Profile
    community: extended:as2-nn4:5:16
    site_id: 1
    peering_type: automatic_with_rr
    peering_password: SomeSecretPeeringPassword
    state: present
  delegate_to: localhost

- name: Query a Fabric External Connection Profile
  cisco.aci.aci_fabric_external_connection_profile:
    host: apic
    username: admin
    password: SomeSecretPassword
    fabric_id: 1
    state: query
  delegate_to: localhost
  register: query_result

- name: Query all Fabric External Connection Profiles
  cisco.aci.aci_fabric_external_connection_profile:
    host: apic
    username: admin
    password: SomeSecretPassword
    state: query
  delegate_to: localhost
  register: query_result

- name: Remove a Fabric External Connection Profile
  cisco.aci.aci_fabric_external_connection_profile:
    host: apic
    username: admin
    password: SomeSecretPassword
    fabric_id: 1
    state: absent
  delegate_to: localhost
"""

RETURN = r"""
current:
  description: The existing configuration from the APIC after the module has finished
  returned: success
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production environment",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
error:
  description: The error information as returned from the APIC
  returned: failure
  type: dict
  sample:
    {
        "code": "122",
        "text": "unknown managed object class foo"
    }
raw:
  description: The raw output returned by the APIC REST API (xml or json)
  returned: parse error
  type: str
  sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class foo"/></imdata>'
sent:
  description: The actual/minimal configuration pushed to the APIC
  returned: info
  type: list
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment"
            }
        }
    }
previous:
  description: The original configuration from the APIC before the module has started
  returned: info
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
proposed:
  description: The assembled configuration from the user-provided parameters
  returned: info
  type: dict
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment",
                "name": "production"
            }
        }
    }
filter_string:
  description: The filter string used for the request
  returned: failure or debug
  type: str
  sample: ?rsp-prop-include=config-only
method:
  description: The HTTP method used for the request to the APIC
  returned: failure or debug
  type: str
  sample: POST
response:
  description: The HTTP response from the APIC
  returned: failure or debug
  type: str
  sample: OK (30 bytes)
status:
  description: The HTTP status from the APIC
  returned: failure or debug
  type: int
  sample: 200
url:
  description: The HTTP url used for the request to the APIC
  returned: failure or debug
  type: str
  sample: https://10.11.12.13/api/mo/uni/tn-production.json
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec, aci_annotation_spec, aci_owner_spec


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(aci_owner_spec())
    argument_spec.update(
        description=dict(type="str", aliases=["descr"]),
        fabric_id=dict(type="int", aliases=["id", "fabric"]),
        name=dict(type="str", aliases=["profile_name"]),
        community=dict(type="str", aliases=["rt", "route_target"]),
        site_id=dict(type="int", aliases=["sid", "site", "s_id"]),
        peering_type=dict(type="str", aliases=["p_type", "peer", "peer_t"], choices=["automatic_with_full_mesh", "automatic_with_rr"]),
        peering_password=dict(type="str", aliases=["peer_password", "peer_pwd"], no_log=True),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["fabric_id"]],
            ["state", "present", ["fabric_id"]],
        ],
    )

    aci = ACIModule(module)

    description = module.params.get("description")
    fabric_id = module.params.get("fabric_id")
    name = module.params.get("name")
    community = module.params.get("community")
    peering_type = module.params.get("peering_type")
    peering_password = module.params.get("peering_password")
    site_id = module.params.get("site_id")
    state = module.params.get("state")

    aci.construct_url(
        root_class=dict(
            aci_class="fvFabricExtConnP",
            aci_rn="tn-infra/fabricExtConnP-{0}".format(fabric_id),
            module_object=fabric_id,
            target_filter={"id": fabric_id},
        ),
        child_classes=["fvPeeringP"],
    )

    aci.get_existing()

    if state == "present":
        child_configs = None
        if peering_type is not None or peering_password is not None:
            peering_p = {"fvPeeringP": {"attributes": {}}}
            if peering_type is not None:
                peering_p["fvPeeringP"]["attributes"]["type"] = peering_type
            if peering_password is not None:
                peering_p["fvPeeringP"]["attributes"]["password"] = peering_password
            child_configs = [peering_p]

        aci.payload(
            aci_class="fvFabricExtConnP",
            class_config=dict(
                descr=description,
                id=fabric_id,
                name=name,
                rt=community,
                siteId=site_id,
            ),
            child_configs=child_configs,
        )

        aci.get_diff(aci_class="fvFabricExtConnP")

        aci.post_config()

    elif state == "absent":
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Copyright (c) 2017 Julien Stroheker, <juliens@microsoft.com>
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: azure_rm_availabilityset

version_added: "0.1.2"

short_description: Manage Azure Availability Set

description:
    - Create, update and delete Azure Availability Set.
    - An availability set cannot be updated, you will have to recreate one instead.
    - The only update operation will be for the tags.

options:
    resource_group:
        description:
            - Name of a resource group where the availability set exists or will be created.
        required: true
        type: str
    name:
        description:
            - Name of the availability set.
        type: str
        required: true
    state:
        description:
            - Assert the state of the availability set.
            - Use C(present) to create or update a availability set and C(absent) to delete a availability set.
        default: present
        type: str
        choices:
            - absent
            - present
    location:
        description:
            - Valid Azure location. Defaults to location of the resource group.
        type: str
    platform_update_domain_count:
        description:
            - Update domains indicate groups of virtual machines and underlying physical hardware that can be rebooted at the same time.
        type: int
        default: 5
    platform_fault_domain_count:
        description:
            - Fault domains define the group of virtual machines that share a common power source and network switch.
            - Should be between C(1) and C(3).
        type: int
        default: 3
    proximity_placement_group:
        description:
            - The proximity placement group that the availability set should be assigned to.
        type: str
    sku:
        description:
            - Define if the availability set supports managed disks.
        default: Classic
        type: str
        choices:
            - Classic
            - Aligned
extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags

author:
    - Julien Stroheker (@julienstroheker)
'''

EXAMPLES = '''
- name: Create an availability set with default options
  azure_rm_availabilityset:
    name: myAvailabilitySet
    location: eastus
    resource_group: myResourceGroup

- name: Create an availability set with advanced options
  azure_rm_availabilityset:
    name: myAvailabilitySet
    location: eastus
    resource_group: myResourceGroup
    platform_update_domain_count: 5
    platform_fault_domain_count: 3
    proximity_placement_group: myProximityPlacementGroup
    sku: Aligned

- name: Delete an availability set
  azure_rm_availabilityset:
    name: myAvailabilitySet
    location: eastus
    resource_group: myResourceGroup
    state: absent
'''

RETURN = '''
state:
    description: Current state of the availability set.
    returned: always
    type: complex
    contains:
        id:
            description:
                - Resource ID.
            type: str
            sample: "/subscriptions/xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx/resourceGroups/v-xisuRG/providers/Microsoft.Compute/availabilitySets/myavailabilityset2"
        location:
            description:
                - Location where the resource lives.
            type: str
            sample: eastus
        name:
            description:
                - Resource name.
            type: str
            sample: myavailabilityset2
        platform_fault_domain_count:
            description:
                - Fault domains values.
            type: int
            sample: 2
        platform_update_domain_count:
            description:
                - Update domains values.
            type: int
            sample: 5
        proximity_placement_group:
            description:
                - The proximity placement group that the availability is assigned to.
            type: str
            sample: myProximityPlacementGroup
        sku:
            description:
                - The availability set supports managed disks.
            type: str
            sample: Aligned
        tags:
            description:
                - Resource tags.
            type: dict
            sample: {env: sandbox}

changed:
    description: Whether or not the resource has changed
    returned: always
    type: bool
    sample: true
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase

try:
    from azure.mgmt.core.tools import parse_resource_id
    from azure.core.exceptions import ResourceNotFoundError
except ImportError:
    # This is handled in azure_rm_common
    pass


def availability_set_to_dict(avaset):
    '''
    Serializing the availability set from the API to Dict
    :return: dict
    '''
    return dict(
        id=avaset.id,
        name=avaset.name,
        location=avaset.location,
        platform_update_domain_count=avaset.platform_update_domain_count,
        platform_fault_domain_count=avaset.platform_fault_domain_count,
        proximity_placement_group=avaset.proximity_placement_group.id if avaset.proximity_placement_group else None,
        tags=avaset.tags,
        sku=avaset.sku.name
    )


class AzureRMAvailabilitySet(AzureRMModuleBase):
    """Configuration class for an Azure RM availability set resource"""

    def __init__(self):
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str',
                required=True
            ),
            name=dict(
                type='str',
                required=True
            ),
            state=dict(
                type='str',
                default='present',
                choices=['present', 'absent']
            ),
            location=dict(
                type='str'
            ),
            platform_update_domain_count=dict(
                type='int',
                default=5
            ),
            platform_fault_domain_count=dict(
                type='int',
                default=3
            ),
            proximity_placement_group=dict(
                type='str',
                required=False
            ),
            sku=dict(
                type='str',
                default='Classic',
                choices=['Classic', 'Aligned']
            )
        )

        self.resource_group = None
        self.name = None
        self.location = None
        self.tags = None
        self.platform_update_domain_count = None
        self.platform_fault_domain_count = None
        self.proximity_placement_group = None
        self.proximity_placement_group_resource = None
        self.sku = None
        self.state = None
        self.warning = False

        self.results = dict(changed=False, state=dict())

        super(AzureRMAvailabilitySet, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                     supports_check_mode=True,
                                                     supports_tags=True)

    def exec_module(self, **kwargs):
        """Main module execution method"""

        for key in list(self.module_arg_spec.keys()) + ['tags']:
            setattr(self, key, kwargs[key])

        resource_group = None
        response = None
        to_be_updated = False
        proximity_placement_group_id = None

        resource_group = self.get_resource_group(self.resource_group)
        if not self.location:
            self.location = resource_group.location

        # Check if the AS already present in the RG
        if self.state == 'present':
            response = self.get_availabilityset()
            self.results['state'] = response

            if self.proximity_placement_group is not None:
                parsed_proximity_placement_group = parse_resource_id(self.proximity_placement_group)
                proximity_placement_group = self.get_proximity_placement_group(parsed_proximity_placement_group.get('resource_group', self.resource_group),
                                                                               parsed_proximity_placement_group.get('name'))
                self.proximity_placement_group_resource = self.compute_models.SubResource(id=proximity_placement_group.id)
                proximity_placement_group_id = proximity_placement_group.id.lower()

            if not response:
                to_be_updated = True
            else:
                update_tags, response['tags'] = self.update_tags(response['tags'])
                response_proximity_placement_group = (
                    response['proximity_placement_group'].lower() if response.get('proximity_placement_group') is not None else None
                )

                if update_tags:
                    self.log("Tags has to be updated")
                    to_be_updated = True

                if response['platform_update_domain_count'] != self.platform_update_domain_count:
                    self.faildeploy('platform_update_domain_count')

                if response['platform_fault_domain_count'] != self.platform_fault_domain_count:
                    self.faildeploy('platform_fault_domain_count')

                if response_proximity_placement_group != proximity_placement_group_id:
                    self.faildeploy('proximity_placement_group')

                if response['sku'] != self.sku:
                    self.faildeploy('sku')

            if self.check_mode:
                self.results['changed'] = to_be_updated
                return self.results

            if to_be_updated:
                self.results['state'] = self.create_or_update_availabilityset()
                self.results['changed'] = True

        elif self.state == 'absent':
            response = self.get_availabilityset()
            if response:
                if not self.check_mode:
                    self.delete_availabilityset()
                self.results['changed'] = True

        return self.results

    def faildeploy(self, param):
        '''
        Helper method to push fail message in the console.
        Useful to notify that the users cannot change some values in a Availability Set

        :param: variable's name impacted
        :return: void
        '''
        self.fail("You tried to change {0} but is was unsuccessful. An Availability Set is immutable, except tags".format(str(param)))

    def create_or_update_availabilityset(self):
        '''
        Method calling the Azure SDK to create or update the AS.
        :return: void
        '''
        self.log("Creating availabilityset {0}".format(self.name))
        try:
            params_sku = self.compute_models.Sku(
                name=self.sku
            )
            params = self.compute_models.AvailabilitySet(
                location=self.location,
                tags=self.tags,
                platform_update_domain_count=self.platform_update_domain_count,
                platform_fault_domain_count=self.platform_fault_domain_count,
                proximity_placement_group=self.proximity_placement_group_resource,
                sku=params_sku
            )
            response = self.compute_client.availability_sets.create_or_update(self.resource_group, self.name, params)
        except Exception as e:
            self.log('Error attempting to create the availability set.')
            self.fail("Error creating the availability set: {0}".format(str(e)))

        return availability_set_to_dict(response)

    def delete_availabilityset(self):
        '''
        Method calling the Azure SDK to delete the AS.
        :return: void
        '''
        self.log("Deleting availabilityset {0}".format(self.name))
        try:
            response = self.compute_client.availability_sets.delete(self.resource_group, self.name)
        except Exception as e:
            self.log('Error attempting to delete the availability set.')
            self.fail("Error deleting the availability set: {0}".format(str(e)))

        return True

    def get_availabilityset(self):
        '''
        Method calling the Azure SDK to get an AS.
        :return: void
        '''
        self.log("Checking if the availabilityset {0} is present".format(self.name))
        found = False
        try:
            response = self.compute_client.availability_sets.get(self.resource_group, self.name)
            found = True
        except ResourceNotFoundError as e:
            self.log('Did not find the Availability set.')
        if found is True:
            return availability_set_to_dict(response)
        else:
            return False

    def get_proximity_placement_group(self, resource_group, name):
        try:
            return self.compute_client.proximity_placement_groups.get(resource_group, name)
        except ResourceNotFoundError as exc:
            self.fail("Error fetching proximity placement group {0} - {1}".format(name, str(exc)))


def main():
    """Main execution"""
    AzureRMAvailabilitySet()


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
# Copyright 2024 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The Vrf_interfaces parser templates file. This contains
a list of parser definitions and associated functions that
facilitates both facts gathering and native command generation for
the given network resource.
"""

import re

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.network_template import (
    NetworkTemplate,
)


class Vrf_interfacesTemplate(NetworkTemplate):
    def __init__(self, lines=None, module=None):
        super(Vrf_interfacesTemplate, self).__init__(lines=lines, tmplt=self, module=module)

    # fmt: off
    PARSERS = [
        {
            'name': 'interface',
            'getval': re.compile(
                r'''
                ^interface
                \s(preconfigure)*\s*
                (?P<name>\S+)$''', re.VERBOSE,
            ),
            'setval': 'interface {{ name }}',
            'result': {
                '{{ name }}': {
                    'name': '{{ name }}',
                },
            },
            'shared': True,
        },
        {
            "name": "vrf_name",
            "getval": re.compile(
                r"""
                \s+vrf\s(?P<vrf_name>\S+)$
                """,
                re.VERBOSE,
            ),
            "setval": "vrf {{ vrf_name }}",
            "result": {
                '{{ name }}': {
                    'vrf_name': '{{ vrf_name }}',
                },
            },
        },
    ]
    # fmt: on



# Copyright (c) Ansible Project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import datetime
import json
import os
import re
import sys
import time
import traceback
import typing as t
from http import HTTPStatus
from urllib.parse import urlencode

from ansible.module_utils.basic import env_fallback, missing_required_lib
from ansible.module_utils.urls import fetch_url

from ansible_collections.community.general.plugins.module_utils._datetime import (
    now,
)

if t.TYPE_CHECKING:
    from collections.abc import Iterable

    from ansible.module_utils.basic import AnsibleModule

SCALEWAY_SECRET_IMP_ERR: str | None = None
try:
    from passlib.hash import argon2

    HAS_SCALEWAY_SECRET_PACKAGE = True
except Exception:
    SCALEWAY_SECRET_IMP_ERR = traceback.format_exc()
    HAS_SCALEWAY_SECRET_PACKAGE = False

YAML_IMPORT_ERROR: str | None
try:
    import yaml
except ImportError:
    YAML_IMPORT_ERROR = traceback.format_exc()
else:
    YAML_IMPORT_ERROR = None


def scaleway_argument_spec() -> dict[str, t.Any]:
    return dict(
        api_token=dict(
            fallback=(env_fallback, ["SCW_TOKEN", "SCW_API_KEY", "SCW_OAUTH_TOKEN", "SCW_API_TOKEN"]),
            no_log=True,
            aliases=["oauth_token"],
        ),
        api_url=dict(
            fallback=(env_fallback, ["SCW_API_URL"]), default="https://api.scaleway.com", aliases=["base_url"]
        ),
        profile=dict(
            fallback=(env_fallback, ["SCW_PROFILE"]),
            aliases=["scw_profile"],
        ),
        api_timeout=dict(type="int", default=30, aliases=["timeout"]),
        query_parameters=dict(type="dict", default={}),
        validate_certs=dict(default=True, type="bool"),
    )


def scaleway_waitable_resource_argument_spec():
    return dict(
        wait=dict(type="bool", default=True),
        wait_timeout=dict(type="int", default=300),
        wait_sleep_time=dict(type="int", default=3),
    )


def payload_from_object(scw_object):
    return {k: v for k, v in scw_object.items() if k != "id" and v is not None}


def get_scw_config_path(scw_profile: str) -> str | None:
    if "SCW_CONFIG_PATH" in os.environ:
        scw_config_path = os.getenv("SCW_CONFIG_PATH", "")
    elif "XDG_CONFIG_HOME" in os.environ:
        scw_config_path = os.path.join(os.getenv("XDG_CONFIG_HOME", ""), "scw", "config.yaml")
    else:
        scw_config_path = os.path.join(os.path.expanduser("~"), ".config", "scw", "config.yaml")

    if os.path.exists(scw_config_path):
        with open(scw_config_path) as fh:
            scw_config = yaml.safe_load(fh)
            return scw_config["profiles"][scw_profile].get("secret_key")

    return None


class ScalewayException(Exception):
    def __init__(self, message: str) -> None:
        self.message = message


# Specify a complete Link header, for validation purposes
R_LINK_HEADER = r"""<[^>]+>;\srel="(first|previous|next|last)"
    (,<[^>]+>;\srel="(first|previous|next|last)")*"""
# Specify a single relation, for iteration and string extraction purposes
R_RELATION = r'</?(?P<target_IRI>[^>]+)>; rel="(?P<relation>first|previous|next|last)"'


def parse_pagination_link(header: str) -> dict[str, str]:
    if not re.match(R_LINK_HEADER, header, re.VERBOSE):
        raise ScalewayException("Scaleway API answered with an invalid Link pagination header")
    else:
        relations = header.split(",")
        parsed_relations = {}
        rc_relation = re.compile(R_RELATION)
        for relation in relations:
            match = rc_relation.match(relation)
            if not match:
                raise ScalewayException("Scaleway API answered with an invalid relation in the Link pagination header")
            data = match.groupdict()
            parsed_relations[data["relation"]] = data["target_IRI"]
        return parsed_relations


def filter_sensitive_attributes(container: dict[str, t.Any], attributes: Iterable[str]) -> dict[str, t.Any]:
    """
    WARNING: This function is effectively private, **do not use it**!
    It will be removed or renamed once changing its name no longer triggers a pylint bug.
    """
    for attr in attributes:
        container[attr] = "SENSITIVE_VALUE"

    return container


class SecretVariables:
    @staticmethod
    def ensure_scaleway_secret_package(module: AnsibleModule) -> None:
        if not HAS_SCALEWAY_SECRET_PACKAGE:
            module.fail_json(
                msg=missing_required_lib("passlib[argon2]", url="https://passlib.readthedocs.io/en/stable/"),
                exception=SCALEWAY_SECRET_IMP_ERR,
            )

    @staticmethod
    def dict_to_list(source_dict):
        return [dict(key=k, value=v) for k, v in source_dict.items()]

    @staticmethod
    def list_to_dict(source_list, hashed=False):
        key_value = "hashed_value" if hashed else "value"
        return {var["key"]: var[key_value] for var in source_list}

    @classmethod
    def decode(cls, secrets_list, values_list):
        secrets_dict = cls.list_to_dict(secrets_list, hashed=True)
        values_dict = cls.list_to_dict(values_list, hashed=False)
        for key in values_dict:
            if key in secrets_dict:
                if argon2.verify(values_dict[key], secrets_dict[key]):
                    secrets_dict[key] = values_dict[key]
                else:
                    secrets_dict[key] = secrets_dict[key]

        return cls.dict_to_list(secrets_dict)


def resource_attributes_should_be_changed(target, wished, verifiable_mutable_attributes, mutable_attributes):
    diff = dict()
    for attr in verifiable_mutable_attributes:
        if wished[attr] is not None and target[attr] != wished[attr]:
            diff[attr] = wished[attr]

    if diff:
        return {attr: wished[attr] for attr in mutable_attributes}
    else:
        return diff


class Response:
    def __init__(self, resp, info):
        self.body = None
        if resp:
            self.body = resp.read()
        self.info = info

    @property
    def json(self):
        if not self.body:
            if "body" in self.info:
                return json.loads(self.info["body"])
            return None
        try:
            return json.loads(self.body)
        except ValueError:
            return None

    @property
    def status_code(self):
        return self.info["status"]

    @property
    def ok(self):
        return self.status_code in (
            HTTPStatus.OK,
            HTTPStatus.CREATED,
            HTTPStatus.ACCEPTED,
            HTTPStatus.NO_CONTENT,
        )


class Scaleway:
    def __init__(self, module: AnsibleModule) -> None:
        self.module = module
        oauth_token = self.module.params.get("api_token")
        scw_profile = self.module.params.get("profile")

        if scw_profile:
            if YAML_IMPORT_ERROR is not None:
                self.module.fail_json(
                    msg=missing_required_lib("PyYAML", reason="for scw_profile"), exception=YAML_IMPORT_ERROR
                )
            oauth_token = get_scw_config_path(scw_profile)

        if oauth_token is None:
            self.module.fail_json(
                msg="Either your config profile could not be loaded or you have not provided an api_token."
            )

        self.headers = {
            "X-Auth-Token": oauth_token,
            "User-Agent": self.get_user_agent_string(module),
            "Content-Type": "application/json",
        }
        self.name = None

    def get_resources(self):
        results = self.get(f"/{self.name}")

        if not results.ok:
            api_response = results.json if results.json is not None else results.body
            message = api_response.get("message") if isinstance(api_response, dict) else api_response
            raise ScalewayException(
                f"Error fetching {self.name} ({self.module.params.get('api_url')}/{self.name}) [{results.status_code}: {message}]"
            )

        if results.json is None:
            raise ScalewayException(f"Scaleway API returned an empty or non-JSON response for {self.name}")

        return results.json.get(self.name)

    def _url_builder(self, path, params):
        d = self.module.params.get("query_parameters")
        if params is not None:
            d.update(params)
        query_string = urlencode(d, doseq=True)

        if path[0] == "/":
            path = path[1:]
        return f"{self.module.params.get('api_url')}/{path}?{query_string}"

    def send(self, method, path, data=None, headers=None, params=None):
        url = self._url_builder(path=path, params=params)
        self.warn(url)

        if headers is not None:
            self.headers.update(headers)

        if self.headers["Content-Type"] == "application/json":
            data = self.module.jsonify(data)

        resp, info = fetch_url(
            self.module,
            url,
            data=data,
            headers=self.headers,
            method=method,
            timeout=self.module.params.get("api_timeout"),
        )

        # Exceptions in fetch_url may result in a status -1, the ensures a proper error to the user in all cases
        if info["status"] == -1:
            self.module.fail_json(msg=info["msg"])

        return Response(resp, info)

    @staticmethod
    def get_user_agent_string(module: AnsibleModule) -> str:
        ansible_version = module.ansible_version  # type: ignore # For some reason this isn't documented in AnsibleModule
        return f"ansible {ansible_version} Python {sys.version.split(' ', 1)[0]}"

    def get(self, path, data=None, headers=None, params=None):
        return self.send(method="GET", path=path, data=data, headers=headers, params=params)

    def put(self, path, data=None, headers=None, params=None):
        return self.send(method="PUT", path=path, data=data, headers=headers, params=params)

    def post(self, path, data=None, headers=None, params=None):
        return self.send(method="POST", path=path, data=data, headers=headers, params=params)

    def delete(self, path, data=None, headers=None, params=None):
        return self.send(method="DELETE", path=path, data=data, headers=headers, params=params)

    def patch(self, path, data=None, headers=None, params=None):
        return self.send(method="PATCH", path=path, data=data, headers=headers, params=params)

    def update(self, path, data=None, headers=None, params=None):
        return self.send(method="UPDATE", path=path, data=data, headers=headers, params=params)

    def warn(self, x) -> None:
        self.module.warn(str(x))

    def fetch_state(self, resource):
        self.module.debug(f"fetch_state of resource: {resource['id']}")
        response = self.get(path=f"{self.api_path}/{resource['id']}")

        if response.status_code == HTTPStatus.NOT_FOUND:
            return "absent"

        if not response.ok:
            msg = f"Error during state fetching: ({response.status_code}) {response.json}"
            self.module.fail_json(msg=msg)

        try:
            self.module.debug(f"Resource {resource['id']} in state: {response.json['status']}")
            return response.json["status"]
        except KeyError:
            self.module.fail_json(msg=f"Could not fetch state in {response.json}")

    def fetch_paginated_resources(self, resource_key, **pagination_kwargs):
        response = self.get(path=self.api_path, params=pagination_kwargs)

        if not response.ok:
            self.module.fail_json(
                msg=f"Error getting {resource_key} [{response.status_code}: {response.json['message']}]"
            )

        return response.json[resource_key]

    def fetch_all_resources(self, resource_key, **pagination_kwargs):
        resources = []

        result = [None]
        while len(result) != 0:
            result = self.fetch_paginated_resources(resource_key, **pagination_kwargs)
            resources += result
            if "page" in pagination_kwargs:
                pagination_kwargs["page"] += 1
            else:
                pagination_kwargs["page"] = 2

        return resources

    def wait_to_complete_state_transition(self, resource, stable_states, force_wait=False):
        wait = self.module.params["wait"]

        if not (wait or force_wait):
            return

        wait_timeout = self.module.params["wait_timeout"]
        wait_sleep_time = self.module.params["wait_sleep_time"]

        # Prevent requesting the resource status too soon
        time.sleep(wait_sleep_time)

        start = now()
        end = start + datetime.timedelta(seconds=wait_timeout)

        while now() < end:
            self.module.debug("We are going to wait for the resource to finish its transition")

            state = self.fetch_state(resource)
            if state in stable_states:
                self.module.debug("It seems that the resource is not in transition anymore.")
                self.module.debug(f"load-balancer in state: {self.fetch_state(resource)}")
                break

            time.sleep(wait_sleep_time)
        else:
            self.module.fail_json(msg="Server takes too long to finish its transition")


SCALEWAY_LOCATION = {
    "par1": {
        "name": "Paris 1",
        "country": "FR",
        "api_endpoint": "https://api.scaleway.com/instance/v1/zones/fr-par-1",
        "api_endpoint_vpc": "https://api.scaleway.com/vpc/v1/zones/fr-par-1",
    },
    "EMEA-FR-PAR1": {
        "name": "Paris 1",
        "country": "FR",
        "api_endpoint": "https://api.scaleway.com/instance/v1/zones/fr-par-1",
        "api_endpoint_vpc": "https://api.scaleway.com/vpc/v1/zones/fr-par-1",
    },
    "par2": {
        "name": "Paris 2",
        "country": "FR",
        "api_endpoint": "https://api.scaleway.com/instance/v1/zones/fr-par-2",
        "api_endpoint_vpc": "https://api.scaleway.com/vpc/v1/zones/fr-par-2",
    },
    "EMEA-FR-PAR2": {
        "name": "Paris 2",
        "country": "FR",
        "api_endpoint": "https://api.scaleway.com/instance/v1/zones/fr-par-2",
        "api_endpoint_vpc": "https://api.scaleway.com/vpc/v1/zones/fr-par-2",
    },
    "par3": {
        "name": "Paris 3",
        "country": "FR",
        "api_endpoint": "https://api.scaleway.com/instance/v1/zones/fr-par-3",
        "api_endpoint_vpc": "https://api.scaleway.com/vpc/v1/zones/fr-par-3",
    },
    "ams1": {
        "name": "Amsterdam 1",
        "country": "NL",
        "api_endpoint": "https://api.scaleway.com/instance/v1/zones/nl-ams-1",
        "api_endpoint_vpc": "https://api.scaleway.com/vpc/v1/zones/nl-ams-1",
    },
    "EMEA-NL-EVS": {
        "name": "Amsterdam 1",
        "country": "NL",
        "api_endpoint": "https://api.scaleway.com/instance/v1/zones/nl-ams-1",
        "api_endpoint_vpc": "https://api.scaleway.com/vpc/v1/zones/nl-ams-1",
    },
    "ams2": {
        "name": "Amsterdam 2",
        "country": "NL",
        "api_endpoint": "https://api.scaleway.com/instance/v1/zones/nl-ams-2",
        "api_endpoint_vpc": "https://api.scaleway.com/vpc/v1/zones/nl-ams-2",
    },
    "ams3": {
        "name": "Amsterdam 3",
        "country": "NL",
        "api_endpoint": "https://api.scaleway.com/instance/v1/zones/nl-ams-3",
        "api_endpoint_vpc": "https://api.scaleway.com/vpc/v1/zones/nl-ams-3",
    },
    "waw1": {
        "name": "Warsaw 1",
        "country": "PL",
        "api_endpoint": "https://api.scaleway.com/instance/v1/zones/pl-waw-1",
        "api_endpoint_vpc": "https://api.scaleway.com/vpc/v1/zones/pl-waw-1",
    },
    "EMEA-PL-WAW1": {
        "name": "Warsaw 1",
        "country": "PL",
        "api_endpoint": "https://api.scaleway.com/instance/v1/zones/pl-waw-1",
        "api_endpoint_vpc": "https://api.scaleway.com/vpc/v1/zones/pl-waw-1",
    },
    "waw2": {
        "name": "Warsaw 2",
        "country": "PL",
        "api_endpoint": "https://api.scaleway.com/instance/v1/zones/pl-waw-2",
        "api_endpoint_vpc": "https://api.scaleway.com/vpc/v1/zones/pl-waw-2",
    },
    "waw3": {
        "name": "Warsaw 3",
        "country": "PL",
        "api_endpoint": "https://api.scaleway.com/instance/v1/zones/pl-waw-3",
        "api_endpoint_vpc": "https://api.scaleway.com/vpc/v1/zones/pl-waw-3",
    },
}

SCALEWAY_ENDPOINT = "https://api.scaleway.com"

SCALEWAY_REGIONS = [
    "fr-par",
    "nl-ams",
    "pl-waw",
]

SCALEWAY_ZONES = [
    "fr-par-1",
    "fr-par-2",
    "fr-par-3",
    "nl-ams-1",
    "nl-ams-2",
    "nl-ams-3",
    "pl-waw-1",
    "pl-waw-2",
    "pl-waw-3",
]



#!/usr/bin/python
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: azure_rm_ml_connection_info
version_added: "3.19.0"
short_description: List ML Connection resources
description:
    - List ML Connection resources.
options:
    name:
        description:
            - Name of the connection.
        type: str
        required: false
    populate_secrets:
        description:
            - For API key-based connections, try to populate the returned
              credentials with the actual secret value. Does nothing for
              non-API key-based connections.
        type: bool
        default: false
    resource_group:
        description:
            - Name of resource group.
        required: true
        type: str
    ml_workspace:
        description:
            - Name of the Azure ML workspace.
        required: true
        type: str

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Bill Peck (@p3ck)
'''

EXAMPLES = '''
- name: List Specific ML connection
  azure.azcollection.azure_rm_ml_connection_info:
    name: MyConnection
    resource_group: myResourceGroup
    ml_workspace: myMLWorkspace

- name: List ML connection
  azure.azcollection.azure_rm_ml_connection_info:
    resource_group: myResourceGroup
    ml_workspace: myMLWorkspace
'''

RETURN = '''
ml_connections:
    description:
        - List of ML Connections.
    returned: always
    type: dict
    sample: [
      {
        "creation_context": {
            "created_at": "2026-05-11T20:07:52.118675+00:00",
            "created_by": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
            "created_by_type": "Application",
            "last_modified_at": "2026-05-11T20:07:52.118675+00:00",
            "last_modified_by": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
            "last_modified_by_type": "Application"
        },
        "credentials": {
            "type": "username_password"
        },
        "id": "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/xxxxx-ml-connection/providers/Microsoft.MachineLearningServices/workspaces/workspacexxxxxxxxxx/connections/connectionxxxxxxxxxx",
        "is_shared": true,
        "metadata": {},
        "name": "connectionxxxxxxxxxx",
        "tags": {},
        "target": "https://test-git-feed.com",
        "type": "git"
      }
    ]
'''  # NOQA


try:
    from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ml import MLClientCommon
    from azure.core.exceptions import ResourceNotFoundError
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMMLConnectionInfo(MLClientCommon):
    def __init__(self):

        self.module_arg_spec = dict(
            name=dict(
                type='str',
                required=False,
            ),
            populate_secrets=dict(
                type='bool',
                default=False,
            ),
            resource_group=dict(
                type='str',
                required=True,
            ),
            ml_workspace=dict(
                type='str',
                required=True,
            ),
        )

        self._client = None
        self.ml_registry = None

        self.results = dict(
            ml_connections=[]
        )

        super(AzureRMMLConnectionInfo, self).__init__(self.module_arg_spec,
                                                      supports_tags=False,
                                                      supports_check_mode=True,
                                                      facts_module=True,
                                                      )

    def exec_module(self, **kwargs):

        for key in list(self.module_arg_spec.keys()):
            setattr(self, key, kwargs[key])

        if self.name:
            try:
                result = self.client.connections.get(
                    name=self.name,
                    populate_secrets=self.populate_secrets
                )
                ml_connections = [self.entity_to_dict(result)]
            except ResourceNotFoundError:
                ml_connections = []
        else:
            results = self.client.connections.list(
                populate_secrets=self.populate_secrets
            )
            ml_connections = [self.entity_to_dict(x) for x in results]

        self.results['ml_connections'] = ml_connections
        return self.results


def main():
    AzureRMMLConnectionInfo()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Contributors to the Ansible project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: rds_instance_param_group
version_added: 5.0.0
short_description: manage RDS parameter groups
description:
  - Creates, modifies, and deletes RDS parameter groups.
  - This module was originally added to C(community.aws) in release 1.0.0.
options:
  state:
    description:
      - Specifies whether the group should be present or absent.
    required: true
    choices: [ 'present' , 'absent' ]
    type: str
  name:
    description:
      - Database parameter group identifier.
    required: true
    type: str
  description:
    description:
      - Database parameter group description. Only set when a new group is added.
    type: str
  engine:
    description:
      - The type of database for this group.
      - Please use M(amazon.aws.rds_engine_versions_info) to get list of all supported db engines and their respective versions.
      - The DB parameter group family is immutable and can't be changed when updating a DB parameter group.
        See U(https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-rds-dbparametergroup.html)
      - Required for O(state=present).
    type: str
  immediate:
    description:
      - Whether to apply the changes immediately, or after the next reboot of any associated instances.
    aliases:
      - apply_immediately
    type: bool
  params:
    description:
      - Map of parameter names and values. Numeric values may be represented as K for kilo (1024), M for mega (1024^2), G for giga (1024^3),
        or T for tera (1024^4), and these values will be expanded into the appropriate number before being set in the parameter group.
    aliases: [parameters]
    type: dict
author:
  - "Scott Anderson (@tastychutney)"
  - "Will Thames (@willthames)"
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.tags
  - amazon.aws.boto3
"""

EXAMPLES = r"""
- name: Add or change a parameter group, in this case setting auto_increment_increment to 42 * 1024
  amazon.aws.rds_instance_param_group:
      state: present
      name: norwegian-blue
      description: 'My Fancy Ex Parrot Group'
      engine: 'mysql5.6'
      params:
          auto_increment_increment: "42K"
      tags:
          Environment: production
          Application: parrot

- name: Remove a parameter group
  amazon.aws.rds_instance_param_group:
      state: absent
      name: norwegian-blue
"""

RETURN = r"""
db_parameter_group_name:
    description: Name of DB parameter group.
    type: str
    returned: when O(state=present)
db_parameter_group_family:
    description: DB parameter group family that this DB parameter group is compatible with.
    type: str
    returned: when O(state=present)
db_parameter_group_arn:
    description: ARN of the DB parameter group.
    type: str
    returned: when O(state=present)
description:
    description: description of the DB parameter group.
    type: str
    returned: when O(state=present)
errors:
    description: List of errors from attempting to modify parameters that are not modifiable.
    type: list
    returned: when O(state=present)
tags:
    description: A dictionary of tags.
    type: dict
    returned: when O(state=present)
"""

from itertools import zip_longest

try:
    import botocore
except ImportError:
    pass  # Handled by AnsibleAWSModule

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict
from ansible.module_utils.parsing.convert_bool import BOOLEANS_TRUE

from ansible_collections.amazon.aws.plugins.module_utils.botocore import is_boto3_error_code
from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry
from ansible_collections.amazon.aws.plugins.module_utils.tagging import ansible_dict_to_boto3_tag_list
from ansible_collections.amazon.aws.plugins.module_utils.tagging import boto3_tag_list_to_ansible_dict
from ansible_collections.amazon.aws.plugins.module_utils.tagging import compare_aws_tags

INT_MODIFIERS = {
    "K": 1024,
    "M": pow(1024, 2),
    "G": pow(1024, 3),
    "T": pow(1024, 4),
}


@AWSRetry.jittered_backoff()
def _describe_db_parameters(connection, **params):
    try:
        paginator = connection.get_paginator("describe_db_parameters")
        return paginator.paginate(**params).build_full_result()
    except is_boto3_error_code("DBParameterGroupNotFound"):
        return None


def convert_parameter(param, value):
    """
    Allows setting parameters with 10M = 10* 1024 * 1024 and so on.
    """
    converted_value = value

    if param["DataType"] == "integer":
        if isinstance(value, str):
            try:
                for name, modifier in INT_MODIFIERS.items():
                    if value.endswith(name):
                        converted_value = int(value[:-1]) * modifier
            except ValueError:
                # may be based on a variable (ie. {foo*3/4}) so
                # just pass it on through to the AWS SDK
                pass
        elif isinstance(value, bool):
            converted_value = 1 if value else 0

    elif param["DataType"] == "boolean":
        if isinstance(value, str):
            converted_value = value in BOOLEANS_TRUE
        # convert True/False to 1/0
        converted_value = 1 if converted_value else 0
    return str(converted_value)


def update_parameters(module, connection):
    groupname = module.params["name"]
    desired = module.params["params"]
    apply_method = "immediate" if module.params["immediate"] else "pending-reboot"
    errors = []
    modify_list = []
    existing = {}
    try:
        _existing = _describe_db_parameters(connection, DBParameterGroupName=groupname)
        if _existing:
            existing = _existing["Parameters"]
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Failed to describe existing parameter groups")
    lookup = dict((param["ParameterName"], param) for param in existing)
    for param_key, param_value in desired.items():
        if param_key not in lookup:
            errors.append(
                f"Parameter {param_key} is not an available parameter for the {module.params.get('engine')} engine"
            )
        else:
            converted_value = convert_parameter(lookup[param_key], param_value)
            # engine-default parameters do not have a ParameterValue, so we'll always override those.
            if converted_value != lookup[param_key].get("ParameterValue"):
                if lookup[param_key]["IsModifiable"]:
                    modify_list.append(
                        dict(ParameterValue=converted_value, ParameterName=param_key, ApplyMethod=apply_method)
                    )
                else:
                    errors.append(f"Parameter {param_key} is not modifiable")

    # modify_db_parameters takes at most 20 parameters
    if modify_list and not module.check_mode:
        for modify_slice in zip_longest(*[iter(modify_list)] * 20, fillvalue=None):
            non_empty_slice = [item for item in modify_slice if item]
            try:
                connection.modify_db_parameter_group(
                    aws_retry=True, DBParameterGroupName=groupname, Parameters=non_empty_slice
                )
            except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
                module.fail_json_aws(e, msg="Couldn't update parameters")
        return True, errors
    return False, errors


def update_tags(module, connection, group, tags):
    if tags is None:
        return False
    changed = False

    existing_tags = connection.list_tags_for_resource(aws_retry=True, ResourceName=group["DBParameterGroupArn"])[
        "TagList"
    ]
    to_update, to_delete = compare_aws_tags(
        boto3_tag_list_to_ansible_dict(existing_tags), tags, module.params["purge_tags"]
    )

    if module.check_mode:
        if not to_update and not to_delete:
            return False
        else:
            return True

    if to_update:
        try:
            connection.add_tags_to_resource(
                aws_retry=True,
                ResourceName=group["DBParameterGroupArn"],
                Tags=ansible_dict_to_boto3_tag_list(to_update),
            )
            changed = True
        except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
            module.fail_json_aws(e, msg="Couldn't add tags to parameter group")
    if to_delete:
        try:
            connection.remove_tags_from_resource(
                aws_retry=True, ResourceName=group["DBParameterGroupArn"], TagKeys=to_delete
            )
            changed = True
        except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
            module.fail_json_aws(e, msg="Couldn't remove tags from parameter group")
    return changed


def ensure_present(module, connection):
    groupname = module.params["name"]
    tags = module.params.get("tags")
    changed = False
    errors = []
    try:
        response = connection.describe_db_parameter_groups(aws_retry=True, DBParameterGroupName=groupname)
    except is_boto3_error_code("DBParameterGroupNotFound"):
        response = None
    except botocore.exceptions.ClientError as e:  # pylint: disable=duplicate-except
        module.fail_json_aws(e, msg="Couldn't access parameter group information")
    if not response:
        params = dict(
            DBParameterGroupName=groupname,
            DBParameterGroupFamily=module.params["engine"],
            Description=module.params["description"],
        )
        if tags:
            params["Tags"] = ansible_dict_to_boto3_tag_list(tags)
        if not module.check_mode:
            try:
                response = connection.create_db_parameter_group(aws_retry=True, **params)
                changed = True
            except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
                module.fail_json_aws(e, msg="Couldn't create parameter group")
    else:
        group = response["DBParameterGroups"][0]
        db_parameter_group_family = group["DBParameterGroupFamily"]

        if module.params.get("engine") != db_parameter_group_family:
            module.warn("The DB parameter group family (engine) can't be changed when updating a DB parameter group.")

        if tags:
            changed = update_tags(module, connection, group, tags)

    if module.params.get("params"):
        params_changed, errors = update_parameters(module, connection)
        changed = changed or params_changed

    try:
        response = connection.describe_db_parameter_groups(aws_retry=True, DBParameterGroupName=groupname)
        group = camel_dict_to_snake_dict(response["DBParameterGroups"][0])
    except is_boto3_error_code("DBParameterGroupNotFound"):
        module.exit_json(changed=True, errors=errors)
    except (
        botocore.exceptions.ClientError,
        botocore.exceptions.BotoCoreError,
    ) as e:  # pylint: disable=duplicate-except
        module.fail_json_aws(e, msg="Couldn't obtain parameter group information")
    try:
        tags = connection.list_tags_for_resource(aws_retry=True, ResourceName=group["db_parameter_group_arn"])[
            "TagList"
        ]
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Couldn't obtain parameter group tags")
    group["tags"] = boto3_tag_list_to_ansible_dict(tags)

    module.exit_json(changed=changed, errors=errors, **group)


def ensure_absent(module, connection):
    group = module.params["name"]
    try:
        response = connection.describe_db_parameter_groups(DBParameterGroupName=group)
    except is_boto3_error_code("DBParameterGroupNotFound"):
        module.exit_json(changed=False)
    except botocore.exceptions.ClientError as e:  # pylint: disable=duplicate-except
        module.fail_json_aws(e, msg="Couldn't access parameter group information")

    if response and module.check_mode:
        module.exit_json(changed=True)

    try:
        response = connection.delete_db_parameter_group(aws_retry=True, DBParameterGroupName=group)
        module.exit_json(changed=True)
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Couldn't delete parameter group")


def main():
    argument_spec = dict(
        state=dict(required=True, choices=["present", "absent"]),
        name=dict(required=True),
        engine=dict(),
        description=dict(),
        params=dict(aliases=["parameters"], type="dict"),
        immediate=dict(type="bool", aliases=["apply_immediately"]),
        tags=dict(type="dict", aliases=["resource_tags"]),
        purge_tags=dict(type="bool", default=True),
    )
    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        required_if=[["state", "present", ["description", "engine"]]],
        supports_check_mode=True,
    )

    try:
        conn = module.client("rds", retry_decorator=AWSRetry.jittered_backoff())
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Failed to connect to AWS")

    state = module.params.get("state")
    if state == "present":
        ensure_present(module, conn)
    if state == "absent":
        ensure_absent(module, conn)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-

# Copyright: Contributors to the Ansible project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import annotations

import typing

if typing.TYPE_CHECKING:
    from typing import Dict

from ..waiter import BaseWaiterFactory


class S3WaiterFactory(BaseWaiterFactory):
    """Factory for creating S3-specific waiters."""

    @property
    def _waiter_model_data(self) -> Dict:
        """
        Define waiter models for S3 operations.

        Returns:
            Dictionary containing waiter configurations for S3 operations.
        """
        PATH_BUCKET_KEY_ENABLED = "ServerSideEncryptionConfiguration.Rules[].BucketKeyEnabled"
        data = dict(
            bucket_exists=dict(
                operation="HeadBucket",
                delay=5,
                maxAttempts=20,
                acceptors=[
                    dict(state="success", matcher="status", expected=200),
                    dict(state="success", matcher="status", expected=301),
                    dict(state="success", matcher="status", expected=403),
                ],
            ),
            bucket_not_exists=dict(
                operation="HeadBucket",
                delay=5,
                maxAttempts=60,
                acceptors=[
                    dict(state="success", matcher="status", expected=404),
                ],
            ),
            bucket_versioning_enabled=dict(
                operation="GetBucketVersioning",
                delay=8,
                maxAttempts=25,
                acceptors=[
                    dict(state="success", matcher="path", argument="Status", expected="Enabled"),
                ],
            ),
            bucket_versioning_suspended=dict(
                operation="GetBucketVersioning",
                delay=8,
                maxAttempts=25,
                acceptors=[
                    dict(state="success", matcher="path", argument="Status", expected="Suspended"),
                ],
            ),
            bucket_key_encryption_enabled=dict(
                operation="GetBucketEncryption",
                delay=5,
                maxAttempts=12,
                acceptors=[
                    dict(state="success", matcher="pathAll", argument=PATH_BUCKET_KEY_ENABLED, expected=True),
                ],
            ),
            bucket_key_encryption_disabled=dict(
                operation="GetBucketEncryption",
                delay=5,
                maxAttempts=12,
                acceptors=[
                    dict(state="success", matcher="pathAll", argument=PATH_BUCKET_KEY_ENABLED, expected=False),
                ],
            ),
        )

        return data


waiter_factory = S3WaiterFactory()



#!/usr/bin/python

# Copyright (c) 2017, Eike Frost <ei@kefro.st>
# Copyright (c) 2021, Christophe Gilles <christophe.gilles54@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or
# https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later
from __future__ import annotations

DOCUMENTATION = r"""
module: keycloak_authz_authorization_scope

short_description: Allows administration of Keycloak client authorization scopes using Keycloak API

version_added: 6.6.0

description:
  - This module allows the administration of Keycloak client Authorization Scopes using the Keycloak REST API. Authorization
    Scopes are only available if a client has Authorization enabled.
  - This module requires access to the REST API using OpenID Connect; the user connecting and the realm being used must have
    the requisite access rights. In a default Keycloak installation, admin-cli and an admin user would work, as would a separate
    realm definition with the scope tailored to your needs and a user having the expected roles.
  - The names of module options are snake_cased versions of the camelCase options used by Keycloak. The Authorization Services
    paths and payloads have not officially been documented by the Keycloak project.
    U(https://www.puppeteers.net/blog/keycloak-authorization-services-rest-api-paths-and-payload/).
attributes:
  check_mode:
    support: full
  diff_mode:
    support: full
  action_group:
    version_added: 10.2.0

options:
  state:
    description:
      - State of the authorization scope.
      - On V(present), the authorization scope is created (or updated if it exists already).
      - On V(absent), the authorization scope is removed if it exists.
    choices: ['present', 'absent']
    default: 'present'
    type: str
  name:
    description:
      - Name of the authorization scope to create.
    type: str
    required: true
  display_name:
    description:
      - The display name of the authorization scope.
    type: str
  icon_uri:
    description:
      - The icon URI for the authorization scope.
    type: str
  client_id:
    description:
      - The C(clientId) of the Keycloak client that should have the authorization scope.
      - This is usually a human-readable name of the Keycloak client.
    type: str
    required: true
  realm:
    description:
      - The name of the Keycloak realm the Keycloak client is in.
    type: str
    required: true

extends_documentation_fragment:
  - community.general._keycloak
  - community.general._keycloak.actiongroup_keycloak
  - community.general._attributes

author:
  - Samuli Seppänen (@mattock)
"""

EXAMPLES = r"""
- name: Manage Keycloak file:delete authorization scope
  keycloak_authz_authorization_scope:
    name: file:delete
    state: present
    display_name: File delete
    client_id: myclient
    realm: myrealm
    auth_keycloak_url: http://localhost:8080/auth
    auth_username: keycloak
    auth_password: keycloak
    auth_realm: master
"""

RETURN = r"""
msg:
  description: Message as to what action was taken.
  returned: always
  type: str

end_state:
  description: Representation of the authorization scope after module execution.
  returned: on success
  type: dict
  contains:
    id:
      description: ID of the authorization scope.
      type: str
      returned: when O(state=present)
      sample: a6ab1cf2-1001-40ec-9f39-48f23b6a0a41
    name:
      description: Name of the authorization scope.
      type: str
      returned: when O(state=present)
      sample: file:delete
    display_name:
      description: Display name of the authorization scope.
      type: str
      returned: when O(state=present)
      sample: File delete
    icon_uri:
      description: Icon URI for the authorization scope.
      type: str
      returned: when O(state=present)
      sample: http://localhost/icon.png
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._keycloak import (
    KeycloakAPI,
    KeycloakError,
    get_token,
    keycloak_argument_spec,
)


def main():
    """
    Module execution

    :return:
    """
    argument_spec = keycloak_argument_spec()

    meta_args = dict(
        state=dict(type="str", default="present", choices=["present", "absent"]),
        name=dict(type="str", required=True),
        display_name=dict(type="str"),
        icon_uri=dict(type="str"),
        client_id=dict(type="str", required=True),
        realm=dict(type="str", required=True),
    )

    argument_spec.update(meta_args)

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_one_of=(
            [["token", "auth_realm", "auth_username", "auth_password", "auth_client_id", "auth_client_secret"]]
        ),
        required_together=([["auth_username", "auth_password"]]),
        required_by={"refresh_token": "auth_realm"},
    )

    result = dict(changed=False, msg="", end_state={}, diff=dict(before={}, after={}))

    # Obtain access token, initialize API
    try:
        connection_header = get_token(module.params)
    except KeycloakError as e:
        module.fail_json(msg=str(e))

    kc = KeycloakAPI(module, connection_header)

    # Convenience variables
    state = module.params.get("state")
    name = module.params.get("name")
    display_name = module.params.get("display_name")
    icon_uri = module.params.get("icon_uri")
    client_id = module.params.get("client_id")
    realm = module.params.get("realm")

    # Get the "id" of the client based on the usually more human-readable
    # "clientId"
    cid = kc.get_client_id(client_id, realm=realm)
    if not cid:
        module.fail_json(msg=f"Invalid client {client_id} for realm {realm}")

    # Get current state of the Authorization Scope using its name as the search
    # filter. This returns False if it is not found.
    before_authz_scope = kc.get_authz_authorization_scope_by_name(name=name, client_id=cid, realm=realm)

    # Generate a JSON payload for Keycloak Admin API. This is needed for
    # "create" and "update" operations.
    desired_authz_scope = {}
    desired_authz_scope["name"] = name
    desired_authz_scope["displayName"] = display_name
    desired_authz_scope["iconUri"] = icon_uri

    # Add "id" to payload for modify operations
    if before_authz_scope:
        desired_authz_scope["id"] = before_authz_scope["id"]

    # Ensure that undefined (null) optional parameters are presented as empty
    # strings in the desired state. This makes comparisons with current state
    # much easier.
    for k, v in desired_authz_scope.items():
        if not v:
            desired_authz_scope[k] = ""

    # Do the above for the current state
    if before_authz_scope:
        for k in ["displayName", "iconUri"]:
            if k not in before_authz_scope:
                before_authz_scope[k] = ""

    if before_authz_scope and state == "present":
        changes = False
        for k, v in desired_authz_scope.items():
            if before_authz_scope[k] != v:
                changes = True
                # At this point we know we have to update the object anyways,
                # so there's no need to do more work.
                break

        if changes:
            if module._diff:
                result["diff"] = dict(before=before_authz_scope, after=desired_authz_scope)

            if module.check_mode:
                result["changed"] = True
                result["msg"] = "Authorization scope would be updated"
                module.exit_json(**result)
            else:
                kc.update_authz_authorization_scope(
                    payload=desired_authz_scope, id=before_authz_scope["id"], client_id=cid, realm=realm
                )
                result["changed"] = True
                result["msg"] = "Authorization scope updated"
        else:
            result["changed"] = False
            result["msg"] = "Authorization scope not updated"

        result["end_state"] = desired_authz_scope
    elif not before_authz_scope and state == "present":
        if module._diff:
            result["diff"] = dict(before={}, after=desired_authz_scope)

        if module.check_mode:
            result["changed"] = True
            result["msg"] = "Authorization scope would be created"
            module.exit_json(**result)
        else:
            kc.create_authz_authorization_scope(payload=desired_authz_scope, client_id=cid, realm=realm)
            result["changed"] = True
            result["msg"] = "Authorization scope created"
            result["end_state"] = desired_authz_scope
    elif before_authz_scope and state == "absent":
        if module._diff:
            result["diff"] = dict(before=before_authz_scope, after={})

        if module.check_mode:
            result["changed"] = True
            result["msg"] = "Authorization scope would be removed"
            module.exit_json(**result)
        else:
            kc.remove_authz_authorization_scope(id=before_authz_scope["id"], client_id=cid, realm=realm)
            result["changed"] = True
            result["msg"] = "Authorization scope removed"
    elif not before_authz_scope and state == "absent":
        result["changed"] = False
    else:
        module.fail_json(
            msg=f"Unable to determine what to do with authorization scope {name} of client {client_id} in realm {realm}"
        )

    module.exit_json(**result)


if __name__ == "__main__":
    main()



# Copyright (c) 2025, Ansible Project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
name: onepassword_ssh_key
author:
  - Mohammed Babelly (@mohammedbabelly20)
short_description: Fetch SSH keys stored in 1Password
version_added: "10.3.0"
description:
  - P(community.general.onepassword_ssh_key#lookup) wraps C(op) command line utility to fetch SSH keys from 1Password.
notes:
  - By default, it returns the private key value in PKCS#8 format, unless O(ssh_format=true) is passed.
  - The pluging works only for C(SSHKEY) type items.
options:
  _terms:
    description: Identifier(s) (case-insensitive UUID or name) of item(s) to retrieve.
    required: true
    type: list
    elements: string
  ssh_format:
    description: Output key in SSH format if V(true). Otherwise, outputs in the default format (PKCS#8).
    default: false
    type: bool

extends_documentation_fragment:
  - community.general._onepassword
  - community.general._onepassword.lookup
"""

EXAMPLES = r"""
---
- name: Retrieve the private SSH key from 1Password
  ansible.builtin.debug:
    msg: "{{ lookup('community.general.onepassword_ssh_key', 'SSH Key', ssh_format=true) }}"
"""

RETURN = r"""
_raw:
  description: Private key of SSH keypair.
  type: list
  elements: string
"""
import json

from ansible.errors import AnsibleLookupError
from ansible.plugins.lookup import LookupBase

from ansible_collections.community.general.plugins.lookup.onepassword import (
    OnePass,
    OnePassCLIv2,
)
from ansible_collections.community.general.plugins.plugin_utils._lookup import check_for_wrong_terms


class LookupModule(LookupBase):
    def get_ssh_key(self, out, item_id, ssh_format=False):
        data = json.loads(out)

        if data.get("category") != "SSH_KEY":
            raise AnsibleLookupError(f"Item {item_id} is not an SSH key")

        private_key_field = next(
            (
                field
                for field in data.get("fields", {})
                if field.get("id") == "private_key" and field.get("type") == "SSHKEY"
            ),
            None,
        )
        if not private_key_field:
            raise AnsibleLookupError(f"No private key found for item {item_id}.")

        if ssh_format:
            return private_key_field.get("ssh_formats", {}).get("openssh", {}).get("value", "")
        return private_key_field.get("value", "")

    def run(self, terms, variables=None, **kwargs):
        self.set_options(var_options=variables, direct=kwargs)
        check_for_wrong_terms(self, direct=kwargs)

        ssh_format = self.get_option("ssh_format")
        vault = self.get_option("vault")
        subdomain = self.get_option("subdomain")
        domain = self.get_option("domain", "1password.com")
        username = self.get_option("username")
        secret_key = self.get_option("secret_key")
        master_password = self.get_option("master_password")
        service_account_token = self.get_option("service_account_token")
        account_id = self.get_option("account_id")
        connect_host = self.get_option("connect_host")
        connect_token = self.get_option("connect_token")

        op = OnePass(
            subdomain=subdomain,
            domain=domain,
            username=username,
            secret_key=secret_key,
            master_password=master_password,
            service_account_token=service_account_token,
            account_id=account_id,
            connect_host=connect_host,
            connect_token=connect_token,
            cli_class=OnePassCLIv2,
        )
        op.assert_logged_in()

        return [self.get_ssh_key(op.get_raw(term, vault), term, ssh_format=ssh_format) for term in terms]



# -*- coding: utf-8 -*-
# Copyright 2025 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The ios bfd_interfaces fact class
It is in this file the configuration is collected from the device
for a given resource, parsed, and the facts tree is populated
based on the configuration.
"""

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common import (
    utils,
)

from ansible_collections.cisco.ios.plugins.module_utils.network.ios.argspec.bfd_interfaces.bfd_interfaces import (
    Bfd_interfacesArgs,
)
from ansible_collections.cisco.ios.plugins.module_utils.network.ios.rm_templates.bfd_interfaces import (
    Bfd_interfacesTemplate,
)


class Bfd_interfacesFacts(object):
    """The ios bfd_interfaces facts class"""

    def __init__(self, module, subspec="config", options="options"):
        self._module = module
        self.argument_spec = Bfd_interfacesArgs.argument_spec

    def get_interfaces_data(self, connection):
        return connection.get("show running-config | section ^interface")

    def populate_facts(self, connection, ansible_facts, data=None):
        """Populate the facts for Bfd_interfaces network resource

        :param connection: the device connection
        :param ansible_facts: Facts dictionary
        :param data: previously collected conf

        :rtype: dictionary
        :returns: facts
        """
        facts = {}
        objs = []

        if not data:
            data = self.get_interfaces_data(connection)

        # parse native config using the Bfd_interfaces template
        bfd_interfaces_parser = Bfd_interfacesTemplate(lines=data.splitlines(), module=self._module)
        objs = list(bfd_interfaces_parser.parse().values())

        ansible_facts["ansible_network_resources"].pop("bfd_interfaces", None)

        params = utils.remove_empties(
            bfd_interfaces_parser.validate_config(
                self.argument_spec,
                {"config": objs},
                redact=True,
            ),
        )

        facts["bfd_interfaces"] = params.get("config", [])
        ansible_facts["ansible_network_resources"].update(facts)

        return ansible_facts
